#include <time.h>
#include <string.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "asusapi.h"
#include <sys/syscall.h>
#include <sys/time.h>
#include <errno.h>
#include <stdarg.h>

#define APP_VERSION 1
#define APP_REVISION 0
#define APP_BUILD 0

FILE *AppOutputStream;

int LOG_FILE(char *fmt, ...) {
    struct timeval tv;
    gettimeofday(&tv, NULL);
    char buf[16384];
    memset(buf, 0, 16384);
    va_list list;
    sprintf(buf, "[%d][%lu][%u:%u] ", getpid(), syscall(SYS_gettid), tv.tv_sec, tv.tv_usec);
    va_start(list, fmt);
    vsprintf(buf + strlen(buf), fmt, list);
    //fprintf(stdout, "%s\n", buf);
    if(AppOutputStream != NULL) {
        fprintf(AppOutputStream, "%s", buf);
        fflush(AppOutputStream);
    }
    va_end(list);
    return 0;
}

int LOGFILE_STDOUT(char *fmt, ...) {
    struct timeval tv;
    gettimeofday(&tv, NULL);
    char buf[16384];
    memset(buf, 0, 16384);
    va_list list;
    sprintf(buf, "[%d][%lu][%u:%u] ", getpid(), syscall(SYS_gettid), tv.tv_sec, tv.tv_usec);
    va_start(list, fmt);
    vsprintf(buf + strlen(buf), fmt, list);
    fprintf(stdout, "%s\n", buf);
    if(AppOutputStream != NULL) {
        fprintf(AppOutputStream, "%s", buf);
        fflush(AppOutputStream);
    }
    va_end(list);
    return 0;
}

void disableLog() {
    LOG_FILE("DisableLog() \n");
    char* logpath = "/var/log/enable_debug.setting";
    int iRet = remove(logpath);
    if(iRet == 0) {
        LOG_FILE("DisableLog() delete file success\n");
    } else {
        LOG_FILE("DisableLog() ERROR delete file\n");
    }
    if(AppOutputStream != NULL) {
        fclose(AppOutputStream);
    }
}

void enableLog() {
    LOG_FILE("EnableLog() \n");
    char* logpath = "/var/log/enable_debug.setting";
    FILE *EnableLogFileStream = NULL;
    EnableLogFileStream = fopen(logpath, "w");
    chmod(logpath, 0666);
    if(EnableLogFileStream != NULL) {
        fclose(EnableLogFileStream);
    }
    logpath = "/var/log/asusapi.log";
    FILE *FileStream = NULL;
    FileStream = fopen(logpath, "r");
    if(FileStream == NULL) {
        FileStream = fopen(logpath, "w");
    }
    chmod(logpath, 0666);
    if(FileStream != NULL) {
        fclose(FileStream);
    }

    logpath = "/var/log/asusapiapp.log";

    AppOutputStream = fopen(logpath, "rb");

    chmod(logpath, 0666);

    if(AppOutputStream == NULL) {
        AppOutputStream = fopen(logpath, "w");

        chmod(logpath, 0666);

    } else {
        fseek(AppOutputStream, 0, SEEK_END);
        unsigned long size = (unsigned long)ftell(AppOutputStream);
        fclose(AppOutputStream);
        if(size >= 2500000) {
            AppOutputStream = fopen(logpath, "w");
        } else {
            AppOutputStream = fopen(logpath, "a+");
        }
        chmod(logpath, 0666);
    }

    if (AppOutputStream == NULL) {
        LOGFILE_STDOUT("Cannot create log file in %s; errno:%d\n", logpath, errno);
    } else {
        LOG_FILE("The version of asusapiapp is %d.%d.%d \n", APP_VERSION, APP_REVISION, APP_BUILD);
    }
}

void printHelp() {
    printf("Usage: asusapiapp -h getstring\n");
    printf("Usage: asusapiapp -h getvalue\n");
    printf("Usage: asusapiapp -h setvalue\n");
    printf("Usage: asusapiapp -h watchdog\n");
    printf("Usage: asusapiapp -h gpio\n");
    //printf("Usage: asusapiapp -h systemboot\n");
    printf("Usage: asusapiapp -h i2c\n");
    printf("Usage: asusapiapp -h cm\n");
    printf("Usage: asusapiapp -h led\n");
    printf("Usage: asusapiapp -h vcpcomport\n");
    printf("Usage: asusapiapp -h vcprtc\n");
    printf("Usage: asusapiapp -h vcpwakeup\n");
    printf("Usage: asusapiapp -h vcppowermgmt\n");
    printf("\nThe log files will be in /var/log/asusapi.log and /var/log/asusapiapp.log \n\n");
}

void printCmdHelp(char *cmd) {
    if(strcmp(cmd, "getstring") == 0) {
        printf("Usage: sudo asusapiapp getstring <id>\n\n");
        printf("There are ids used in HW monitor:\n");
        printf("  %d, EAPI_ID_BOARD_MANUFACTURER_STR\n", EAPI_ID_BOARD_MANUFACTURER_STR);
        printf("  %d, EAPI_ID_BOARD_NAME_STR\n", EAPI_ID_BOARD_NAME_STR);
        printf("  %d, EAPI_ID_BOARD_REVISION_STR\n", EAPI_ID_BOARD_REVISION_STR);
        printf("  %d, EAPI_ID_BOARD_SERIAL_STR\n", EAPI_ID_BOARD_SERIAL_STR);
        printf("  %d, EAPI_ID_BOARD_BIOS_REVISION_STR\n", EAPI_ID_BOARD_BIOS_REVISION_STR);
        printf("  %d, EAPI_ID_BOARD_HW_REVISION_STR\n", EAPI_ID_BOARD_HW_REVISION_STR);
        printf("  %d, EAPI_ID_BOARD_PLATFORM_TYPE_STR\n", EAPI_ID_BOARD_PLATFORM_TYPE_STR);
    } else if(strcmp(cmd, "getvalue") == 0) {
        printf("Usage: sudo asusapiapp getvalue <id>\n\n");
        printf("There are ids used in HW monitor:\n");
        printf("  %d, EAPI_ID_GET_EAPI_SPEC_VERSION\n", EAPI_ID_GET_EAPI_SPEC_VERSION);
        printf("  %d, EAPI_ID_BOARD_BOOT_COUNTER_VAL\n", EAPI_ID_BOARD_BOOT_COUNTER_VAL);
        printf("  %d, EAPI_ID_BOARD_RUNNING_TIME_METER_VAL\n", EAPI_ID_BOARD_RUNNING_TIME_METER_VAL);
        printf("  %d, EAPI_ID_BOARD_PNPID_VAL\n", EAPI_ID_BOARD_PNPID_VAL);
        printf("  %d, EAPI_ID_BOARD_PLATFORM_REV_VAL\n", EAPI_ID_BOARD_PLATFORM_REV_VAL);
        printf("  %d, EAPI_ID_BOARD_DRIVER_VERSION_VAL\n", EAPI_ID_BOARD_DRIVER_VERSION_VAL);
        printf("  %d, EAPI_ID_BOARD_LIB_VERSION_VAL\n", EAPI_ID_BOARD_LIB_VERSION_VAL);
        printf("  %d, EAPI_ID_HWMON_CPU_TEMP\n", EAPI_ID_HWMON_CPU_TEMP);
        printf("  %d, EAPI_ID_HWMON_CHIPSET_TEMP\n", EAPI_ID_HWMON_CHIPSET_TEMP);
        printf("  %d, EAPI_ID_HWMON_SYSTEM_TEMP\n", EAPI_ID_HWMON_SYSTEM_TEMP);
        printf("  %d, EAPI_ID_HWMON_VOLTAGE_VCORE\n", EAPI_ID_HWMON_VOLTAGE_VCORE);
        printf("  %d, EAPI_ID_HWMON_VOLTAGE_2V5\n", EAPI_ID_HWMON_VOLTAGE_2V5);
        printf("  %d, EAPI_ID_HWMON_VOLTAGE_3V3\n", EAPI_ID_HWMON_VOLTAGE_3V3);
        printf("  %d, EAPI_ID_HWMON_VOLTAGE_VBAT\n", EAPI_ID_HWMON_VOLTAGE_VBAT);
        printf("  %d, EAPI_ID_HWMON_VOLTAGE_5V\n", EAPI_ID_HWMON_VOLTAGE_5V);
        printf("  %d, EAPI_ID_HWMON_VOLTAGE_5VSB\n", EAPI_ID_HWMON_VOLTAGE_5VSB);
        printf("  %d, EAPI_ID_HWMON_VOLTAGE_12V\n", EAPI_ID_HWMON_VOLTAGE_12V);
        printf("  %d, EAPI_ID_HWMON_FAN_CPU\n", EAPI_ID_HWMON_FAN_CPU);
        printf("  %d, EAPI_ID_HWMON_FAN_SYSTEM\n", EAPI_ID_HWMON_FAN_SYSTEM);
        printf("  %d, ASUS_ID_HWMON_VOLTAGE_VTT\n", ASUS_ID_HWMON_VOLTAGE_VTT);
        printf("  %d, ASUS_ID_HWMON_VOLTAGE_DCIN\n", ASUS_ID_HWMON_VOLTAGE_DCIN);
        printf("  %d, ASUS_ID_HWMON_CURRENT_SYSTEM\n", ASUS_ID_HWMON_CURRENT_SYSTEM);
    } else if(strcmp(cmd, "setvalue") == 0) {
        printf("Usage: sudo asusapiapp setvalue <id> <value>\n\n");
        printf("There are ids used in HW monitor:\n");
        printf("  %d, EAPI_ID_HWMON_FAN_CPU; the unit of the value: RPM\n", EAPI_ID_HWMON_FAN_CPU);
        printf("  %d, EAPI_ID_HWMON_FAN_SYSTEM; the unit of the value: RPM\n", EAPI_ID_HWMON_FAN_SYSTEM);
    } else if(strcmp(cmd, "watchdog") == 0) {
        printf("Usage: sudo asusapiapp watchdog <command> [<args>]\n\n");
        printf("There are commands used in Watchdog Timer:\n");
        printf("  getcap           \t Get the capability of the Watchdog Timer\n");
        printf("  start x y z t    \t Start the Watchdog Timer with x:Delay ms., y:Event Timeout ms., and z:Reset Timeout ms. In every t ms., the trigger will be generated. \n");
    } else if(strcmp(cmd, "gpio") == 0) {
        printf("Usage: sudo asusapiapp gpio <command> <id> [<args>]\n\n");
        printf("There are commands used in GPIO:\n");
        printf("  getcap <id>           \t Get the capability of <id>\n");
        printf("  getdirection <id> x   \t Get direction of <id> with bit mask x\n");
        printf("  getlevel <id> x       \t Get level of <id> with bit mask x\n");
        printf("  setdirection <id> x y \t Set direction y on <id> with bit mask x, which y=0 means output and y=1 means input\n");
        printf("  setlevel <id> x y     \t Set level y on <id> with bit mask x, which y=0 means low and y=1 means high\n");
        printf("There are <id> used in GPIO:\n");
        printf("  0-95, single port addressing; EX. sudo asusapiapp gpio getcap 0 \n");
        printf("  65536, bank addressing(GPIOs 0-31); EX. sudo asusapiapp gpio getcap 65536 \n");
        printf("  65537, bank addressing(GPIOs 32-63) \n");
        printf("  65538, bank addressing(GPIOs 64-95) \n");
    } else if(strcmp(cmd, "led") == 0) {
        printf("Usage: sudo asusapiapp led <command> [<args>]\n\n");
        printf("There are commands used in LED:\n");
        printf("  getnumberofleds  \t Get the number of the LEDs\n");
        printf("  getcap           \t Get the capability of the LEDs\n");
        printf("  setoccupied x y  \t Set the occupied flag y on LED x\n");
        printf("    If y=0, the system will not occupy the control of LED x.\n");
        printf("    If y=1, the system will occupy the the control of LED x.\n");
        printf("  turnon x y       \t Turn LED x on with the color y\n");
        printf("  turnoff x        \t Turn LED x off\n");
    } else if(strcmp(cmd, "systemboot") == 0) {
        printf("Usage: sudo asusapiapp systemboot <command> <id> [<args>]\n\n");
        printf("There are commands used in System Boot:\n");
        printf("  get <id>                                                             \t Get the information of the System Boot <id>\n");
        printf("  set <id> <freq> <dayofmonth> <dayofweek> <hour> <minute> <second>    \t Set the information of the System Boot <id>\n");
        printf("There are <id> used in System Boot:\n");
        printf("  %d, ASUS_ID_SYSTEM_BOOT_BOOTUP\n", ASUS_ID_SYSTEM_BOOT_BOOTUP);
        printf("  %d, ASUS_ID_SYSTEM_BOOT_SHUTDOWN\n", ASUS_ID_SYSTEM_BOOT_SHUTDOWN);
        printf("  %d, ASUS_ID_SYSTEM_BOOT_SLEEP\n", ASUS_ID_SYSTEM_BOOT_SLEEP);
        printf("  %d, ASUS_ID_SYSTEM_BOOT_RESTART\n", ASUS_ID_SYSTEM_BOOT_RESTART);
        printf("There are <freq> used in System Boot:\n");
        printf("  %d, ASUS_SYSTEM_BOOT_DISABLE_FREQUENCY\n", ASUS_SYSTEM_BOOT_DISABLE_FREQUENCY);
        printf("  %d, ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_DAILY\n", ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_DAILY);
        printf("  %d, ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_WEEKLY\n", ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_WEEKLY);
        printf("  %d, ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_MONTHLY\n", ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_MONTHLY);
        printf("There are <dayofmonth> used in System Boot:\n");
        printf("  1-31, which means 1st, 2nd, ..., or 31th of month\n");
        printf("There are <dayofweek> used in System Boot:\n");
        printf("  %d, ASUS_SYSTEM_BOOT_WEEKLY_SUNDAY\n", ASUS_SYSTEM_BOOT_WEEKLY_SUNDAY);
        printf("  %d, ASUS_SYSTEM_BOOT_WEEKLY_MONDAY\n", ASUS_SYSTEM_BOOT_WEEKLY_MONDAY);
        printf("  %d, ASUS_SYSTEM_BOOT_WEEKLY_TUESDAY\n", ASUS_SYSTEM_BOOT_WEEKLY_TUESDAY);
        printf("  %d, ASUS_SYSTEM_BOOT_WEEKLY_WEDNESDAY\n", ASUS_SYSTEM_BOOT_WEEKLY_WEDNESDAY);
        printf("  %d, ASUS_SYSTEM_BOOT_WEEKLY_THURSDAY\n", ASUS_SYSTEM_BOOT_WEEKLY_THURSDAY);
        printf("  %d, ASUS_SYSTEM_BOOT_WEEKLY_FRIDAY\n", ASUS_SYSTEM_BOOT_WEEKLY_FRIDAY);
        printf("  %d, ASUS_SYSTEM_BOOT_WEEKLY_SATURDAY\n", ASUS_SYSTEM_BOOT_WEEKLY_SATURDAY);
        printf("There are <hour> used in System Boot:\n");
        printf("  0-23, according to 24-hour clock\n");
        printf("There are <minute> used in System Boot:\n");
        printf("  0-59, the unit of the value: minute\n");
        printf("There are <second> used in System Boot:\n");
        printf("  0-59, the unit of the value: second\n");
    } else if(strcmp(cmd, "cm") == 0) {
        printf("Usage: sudo asusapiapp cm <command> [<args>]\n\n");
        printf("There are commands used in connectivity management:\n");
        printf("  getmodeminfo                  \t Get the <index> and other information of the modems.\n");
        printf("  getnumberofmodems             \t Get the number of the modems.\n");
        printf("  startnetwork <index>          \t Start the network connectivity of modem with <index>.\n");
        printf("  stopnetwork <index>           \t Stop the network connectivity of modem with <index>.\n");
        printf("  poweron <index>               \t Power on modem with <index>.\n");
        printf("  poweroff <index>              \t Power off modem with <index>.\n");
        printf("  restart <index>               \t Power off and power on modem with <index>.\n");
        printf("  getattachedstatus <index>     \t Get the attached status of modem with <index>.\n");
        printf("  getstatus <index>             \t Get the status of the network connection and the info. of IP of modem with <index>.\n");
        printf("  switchsim <index> x           \t Switch to the SIM slot x on modem with <index>.\n");
        printf("  unlockpin <index> x           \t Unlock the SIM of modem with <index> by the PIN code x.\n");
        printf("  setflightmode <index> x       \t Turn on or turn off the flight mode of modem with <index>. x=0 for OFF; x=1 for ON.\n");
        printf("  setapn x                      \t Set APN to x.\n");
        printf("  setuser x                     \t Set the user name to x.\n");
        printf("  setpassword x                 \t Set the password to x.\n");
        printf("  setiptype x                   \t Set the IP type to x. x=0 for IPv4; x=1 for IPv6; x=2 for IPv4/IPv6.\n");
        printf("  getprofile                    \t Get the profile.\n");
        printf("  resetprofile                  \t Set the profile to the default value.\n");
        printf("  setkeepalive x                \t Enable or disable the keep alive feature; x=1 for enable; x=0 for disable.\n");
        printf("  getkeepalivestatus            \t Get the status of the keep alive feature.\n");
        printf("  checkcarrier <index>          \t Get the info. of the carrier on modem with <index>.\n");
        printf("  switchcarrier <index> x y     \t Switch to the carrier with MCC x and MNC y on modem with <index>.\n");
        printf("  geticcid <index>              \t Get ICCID of modem with <index>.\n");
        printf("  getimsi <index>               \t Get IMSI of modem with <index>.\n");
        printf("  getsignalstrength <index>     \t Get the percentage of the signal strength of modem with <index>.\n");
        printf("  getcelllocationinfo <index>   \t Get the cell location info of modem with <index>.\n");
        printf("  getadvancedsignalinfo <index> \t Get the signal strength of the different measurement of modem with <index>.\n");
        printf("  setfailover x                 \t Enable or disable the failover feature; x=1 for enable; x=0 for disable.\n");
        printf("  getfailoverstatus             \t Get the status of the failover feature.\n");
        printf("  setfailovergroup x1 x2 ... xn \t Set the failover group x1, x2, .., xn on the failover feature.\n");
        printf("  getfailovergroup              \t Get the failover group of the failover feature.\n");
        printf("  cleanfailovergroup            \t Clean the failover group on the failover feature.\n");
    } else if(strcmp(cmd, "i2c") == 0) {
        printf("Usage: sudo asusapiapp i2c <command> <i2c bus id> [<args>]\n\n");
        
        printf("There are commands used in I2C:\n\n");
        
        printf("1. getbuscap <i2c bus id>\n");
        printf("  Gets the capabilities of the selected I2C bus.\n");
        printf("  EX. sudo asusapiapp i2c getbuscap 0x03\n");
        printf("  The above example means to get the capabilities of the I2C bus 0x03.\n\n");

        printf("2. probedevice <i2c bus id> <address>\n");
        printf("  Probes I2C address to test I2C device present.\n");
        printf("  EX. sudo asusapiapp i2c probedevice 0x03 0x10\n");
        printf("  The above example means to probe I2C address 0x10 to test I2C bus 0x03.\n\n");

        printf("3. readtransfer <i2c bus id> <address> <i2c command/offset> <byte count to read>\n");
        printf("  Reads from a specific register in the selected I2C device.\n");
        printf("  EX. sudo asusapiapp i2c readtransfer 0x03 0x10 0x03 0x21\n");
        printf("  The above example means to read 0x21 bytes from 0x03 command/offset in the I2C bus 0x03 with address 0x10.\n\n");

        printf("4. writetransfer <i2c bus id> <address> <i2c command/offset> <write data> <byte count to write>\n");
        printf("  Writes to a specific register in the selected I2C device.\n");
        printf("  EX. sudo asusapiapp i2c writetransfer 0x03 0x10 0x03 0x03 0x01\n");
        printf("  The above example means to write the 1-byte data 0x03 from 0x03 command/offset in the I2C bus 0x03 with address 0x10.\n\n");

        printf("5. readraw <i2c bus id> <address> <number of bytes to read plus 1>\n");
        printf("  Universal function for read operations to the I2C bus.\n");
        printf("  EX. sudo asusapiapp i2c readraw 0x03 0x10 0x22\n");
        printf("  The above example means to read 0x21 bytes in the I2C bus 0x03 with address 0x10.\n\n");

        printf("6. writeraw <i2c bus id> <address> <write data> <number of bytes to write plus 1>\n");
        printf("  Universal function for write operations to the I2C bus.\n");
        printf("  EX. sudo asusapiapp i2c writeraw 0x03 0x10 0x03 0x02\n");
        printf("  The above example means to write the 1-byte data 0x03 in the I2C bus 0x03 with address 0x10.\n\n");

        printf("7. writereadraw <i2c bus id> <address> <write data> <number of bytes to write plus 1> <number of bytes to read plus 1>\n");
        printf("  Universal function for read and write operations to the I2C bus.\n");
        printf("  EX. sudo asusapiapp i2c writereadraw 0x03 0x10 0x03 0x02 0x22\n");
        printf("  The above example means to write the 1-byte data 0x03 and reads 0x21 bytes, in the I2C bus 0x03 with address 0x10.\n\n");
    } else if(strcmp(cmd, "vcprtc") == 0) {
        printf("Usage: sudo asusapiapp vcprtc <command> [<args>]\n\n");
        printf("There are commands used in VCP RTC:\n");
        printf("  get                                                                  \t Get the RTC of the VCP.\n");
        printf("  set <year> <month> <dayofmonth> <hour> <minute> <second> <dayofweek> \t Set the RTC of the VCP.\n");
        printf("There are <year> used in VCP RTC:\n");
        printf("  2000-2099, the unit of the value: A.D.\n");
        printf("There are <month> used in VCP RTC:\n");
        printf("  1-12, which means Jan., Feb., ..., or Dec.\n");
        printf("There are <dayofmonth> used in VCP RTC:\n");
        printf("  1-31, which means 1st, 2nd, ..., or 31th of month\n");
        printf("There are <hour> used in VCP RTC:\n");
        printf("  0-23, according to 24-hour clock\n");
        printf("There are <minute> used in VCP RTC:\n");
        printf("  0-59, the unit of the value: minute\n");
        printf("There are <second> used in VCP RTC:\n");
        printf("  0-59, the unit of the value: second\n");
        printf("There are <dayofweek> used in VCP RTC:\n");
        printf("  1-7, which means Monday, Tuesday, ..., or Sunday\n");
    } else if(strcmp(cmd, "vcpwakeup") == 0) {
        printf("Usage: sudo asusapiapp vcpwakeup <command> [<args>]\n\n");
        printf("There are commands used in VCP Wakeup:\n");
        printf("  getenabledwakeuptype  \t Get the enabled wakeup types.\n");
        printf("  getlastwakeupsource   \t Get the last wakeup source.\n");
        printf("  enable <wakeup type>  \t Enable the wakeup type.\n");
        printf("  disable <wakeup type> \t Disable the wakeup type.\n");
        printf("There are <wakeup type> used in the operation:\n");
        printf("  %d, VCP_WAKEUP_TYPE_DI0\n", VCP_WAKEUP_TYPE_DI0);
        printf("  %d, VCP_WAKEUP_TYPE_DI1\n", VCP_WAKEUP_TYPE_DI1);
        printf("  %d, VCP_WAKEUP_TYPE_DI2\n", VCP_WAKEUP_TYPE_DI2);
        printf("  %d, VCP_WAKEUP_TYPE_DI3\n", VCP_WAKEUP_TYPE_DI3);
        printf("  %d, VCP_WAKEUP_TYPE_IGNITION_OFF\n", VCP_WAKEUP_TYPE_IGNITION_OFF);
        printf("  %d, VCP_WAKEUP_TYPE_IGNITION_ON\n", VCP_WAKEUP_TYPE_IGNITION_ON);
        printf("  %d, VCP_WAKEUP_TYPE_GSENSOR\n", VCP_WAKEUP_TYPE_GSENSOR);
        printf("  %d, VCP_WAKEUP_TYPE_LTE\n", VCP_WAKEUP_TYPE_LTE);
        printf("  %d, VCP_WAKEUP_TYPE_PMIC\n", VCP_WAKEUP_TYPE_PMIC);
    } else if(strcmp(cmd, "vcpcomport") == 0) {
        printf("Usage: sudo asusapiapp vcpcomport <command> <com port id> [<args>]\n\n");
        printf("There are commands used in VCP Com Port:\n");
        printf("  enable <com port id>                                                                 \t Enable the selected com port.\n");
        printf("  disable <com port id>                                                                \t Disable the selected com port.\n");
        printf("  read <com port id> <byte count to read>                                              \t Read from the selected com port.\n");
        printf("  write <com port id> <write data> <byte count to write>                               \t Write to the selected com port.\n");
        printf("  init <com port id> <mode> <baudrate> <parity> <datawidth> <stopbit> <flowcontrol>    \t Initialize the selected com port.\n");
        printf("There are <mode> used in the operation:\n");
        printf("  %d, VCP_COM_PORT_MODE_RS232\n", VCP_COM_PORT_MODE_RS232);
        printf("  %d, VCP_COM_PORT_MODE_RS422\n", VCP_COM_PORT_MODE_RS422);
        printf("  %d, VCP_COM_PORT_MODE_RS485\n", VCP_COM_PORT_MODE_RS485);
        printf("There are <baudrate> used in the operation:\n");
        printf("  %d, VCP_COM_PORT_BAUD_RATE_2400\n", VCP_COM_PORT_BAUD_RATE_2400);
        printf("  %d, VCP_COM_PORT_BAUD_RATE_4800\n", VCP_COM_PORT_BAUD_RATE_4800);
        printf("  %d, VCP_COM_PORT_BAUD_RATE_9600\n", VCP_COM_PORT_BAUD_RATE_9600);
        printf("  %d, VCP_COM_PORT_BAUD_RATE_19200\n", VCP_COM_PORT_BAUD_RATE_19200);
        printf("  %d, VCP_COM_PORT_BAUD_RATE_38400\n", VCP_COM_PORT_BAUD_RATE_38400);
        printf("  %d, VCP_COM_PORT_BAUD_RATE_57600\n", VCP_COM_PORT_BAUD_RATE_57600);
        printf("  %d, VCP_COM_PORT_BAUD_RATE_115200\n", VCP_COM_PORT_BAUD_RATE_115200);
        printf("There are <parity> used in the operation:\n");
        printf("  %d, VCP_COM_PORT_PARITY_NONE\n", VCP_COM_PORT_PARITY_NONE);
        printf("  %d, VCP_COM_PORT_PARITY_EVEN\n", VCP_COM_PORT_PARITY_EVEN);
        printf("  %d, VCP_COM_PORT_PARITY_ODD\n", VCP_COM_PORT_PARITY_ODD);
        printf("There are <datawidth> used in the operation:\n");
        printf("  %d, VCP_COM_PORT_DATA_7_BIT\n", VCP_COM_PORT_DATA_7_BIT);
        printf("  %d, VCP_COM_PORT_DATA_8_BIT\n", VCP_COM_PORT_DATA_8_BIT);
        printf("  %d, VCP_COM_PORT_DATA_9_BIT\n", VCP_COM_PORT_DATA_9_BIT);
        printf("There are <stopbit> used in the operation:\n");
        printf("  %d, VCP_COM_PORT_STOP_0_5_BIT\n", VCP_COM_PORT_STOP_0_5_BIT);
        printf("  %d, VCP_COM_PORT_STOP_1_BIT\n", VCP_COM_PORT_STOP_1_BIT);
        printf("  %d, VCP_COM_PORT_STOP_1_5_BIT\n", VCP_COM_PORT_STOP_1_5_BIT);
        printf("  %d, VCP_COM_PORT_STOP_2_BIT\n", VCP_COM_PORT_STOP_2_BIT);
        printf("There are <flowcontrol> used in the operation:\n");
        printf("  %d, VCP_COM_PORT_FLOW_CONTROL_DISABLED\n", VCP_COM_PORT_FLOW_CONTROL_DISABLED);
        printf("  %d, VCP_COM_PORT_FLOW_CONTROL_ENABLED\n", VCP_COM_PORT_FLOW_CONTROL_ENABLED);
    } else if(strcmp(cmd, "vcppowermgmt") == 0) {
        printf("Usage: sudo asusapiapp vcppowermgmt <command> [<args>]\n\n");
        printf("There are commands used in VCP Power Management:\n");
        printf("  setlbpthreshold x             \t Set the voltage x of the threshold of the low battery protection.\n");
        printf("  getlbpthreshold               \t Get the voltage of the threshold of the low battery protection.\n");
        printf("  seteventdelay <event type> x  \t Set x seconds of the delay on <event type>.\n");
        printf("  geteventdelay <event type>    \t Get seconds of the delay on <event type>.\n");
        printf("There are <event type> used in the operation:\n");
        printf("  %d, VCP_POWER_MGMT_EVENT_POWER_ON\n", VCP_POWER_MGMT_EVENT_POWER_ON);
        printf("  %d, VCP_POWER_MGMT_EVENT_POWER_OFF\n", VCP_POWER_MGMT_EVENT_POWER_OFF);
    } else {
        printHelp();
    }
}

void parseErrorCode(EApiStatus_t err) {
    if (err == ASUS_API_STATUS_NOT_INITIALIZED || err == EAPI_STATUS_NOT_INITIALIZED) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_NOT_INITIALIZED\n");
    } else if(err == ASUS_API_STATUS_INITIALIZED || err == EAPI_STATUS_INITIALIZED) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_INITIALIZED\n");
    } else if(err == ASUS_API_STATUS_ALLOC_ERROR || err == EAPI_STATUS_ALLOC_ERROR) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_ALLOC_ERROR\n");
    } else if(err == ASUS_API_STATUS_DRIVER_TIMEOUT || err == EAPI_STATUS_DRIVER_TIMEOUT) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_DRIVER_TIMEOUT\n");
    } else if(err == ASUS_API_STATUS_DEVICE_NOT_READY || err == EAPI_STATUS_DEVICE_NOT_READY) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_DEVICE_NOT_READY\n");
    } else if(err == ASUS_API_STATUS_INVALID_PARAMETER || err == EAPI_STATUS_INVALID_PARAMETER) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_INVALID_PARAMETER\n");
    } else if(err == ASUS_API_STATUS_INVALID_BLOCK_ALIGNMENT || err == EAPI_STATUS_INVALID_BLOCK_ALIGNMENT) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_INVALID_BLOCK_ALIGNMENT\n");
    } else if(err == ASUS_API_STATUS_INVALID_BLOCK_LENGTH || err == EAPI_STATUS_INVALID_BLOCK_LENGTH) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_INVALID_BLOCK_LENGTH\n");
    } else if(err == ASUS_API_STATUS_INVALID_DIRECTION || err == EAPI_STATUS_INVALID_DIRECTION) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_INVALID_DIRECTION\n");
    } else if(err == ASUS_API_STATUS_INVALID_BITMASK || err == EAPI_STATUS_INVALID_BITMASK) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_INVALID_BITMASK\n");
    } else if(err == ASUS_API_STATUS_RUNNING || err == EAPI_STATUS_RUNNING) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_RUNNING\n");
    } else if(err == ASUS_API_STATUS_UNSUPPORTED || err == EAPI_STATUS_UNSUPPORTED) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_UNSUPPORTED\n");
    } else if(err == ASUS_API_STATUS_NOT_FOUND || err == EAPI_STATUS_NOT_FOUND) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_NOT_FOUND\n");
    } else if(err == ASUS_API_STATUS_TIMEOUT || err == EAPI_STATUS_TIMEOUT) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_TIMEOUT\n");
    } else if(err == ASUS_API_STATUS_BUSY_COLLISION || err == EAPI_STATUS_BUSY_COLLISION) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_BUSY_COLLISION\n");
    } else if(err == ASUS_API_STATUS_READ_ERROR || err == EAPI_STATUS_READ_ERROR) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_READ_ERROR\n");
    } else if(err == ASUS_API_STATUS_WRITE_ERROR || err == EAPI_STATUS_WRITE_ERROR) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_WRITE_ERROR\n");
    } else if(err == ASUS_API_STATUS_MORE_DATA || err == EAPI_STATUS_MORE_DATA) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_MORE_DATA\n");
    } else if(err == ASUS_API_STATUS_INVALID_PERMISSION_ACCESS) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_INVALID_PERMISSION_ACCESS\n");
    } else if(err == ASUS_API_STATUS_ERROR || err == EAPI_STATUS_ERROR) {
        LOGFILE_STDOUT ("ASUS_API_STATUS_ERROR\n");
    } else {
        LOGFILE_STDOUT ("Error Code: 0x%02x\n", err);
    }
}

/**
 * hex2int
 * take a hex string and convert it to a 32bit number (max 8 hex digits)
 */
uint32_t hex2int(char *hex) {
    uint32_t val = 0;
    while (*hex) {
        // get current character then increment
        uint8_t byte = *hex++; 
        // transform hex character to the 4bit equivalent number, using the ascii table indexes
        if (byte >= '0' && byte <= '9') byte = byte - '0';
        else if (byte >= 'a' && byte <='f') byte = byte - 'a' + 10;
        else if (byte >= 'A' && byte <='F') byte = byte - 'A' + 10;    
        // shift 4 to make space for new digit, and add the 4 bits of the new digit 
        val = (val << 4) | (byte & 0xF);
    }
    return val;
}

int eapiGetString(EApiId_t id) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    char buf[1024];
    uint32_t lenBuf = 1024;
    
    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiGetString() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eapiGetString() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eapiGetString() eApiLibInitialize return success\n");
    }

    StatusCode = EApiBoardGetStringA(id, buf, &lenBuf);
    if(StatusCode == EAPI_STATUS_SUCCESS) {
        LOG_FILE("%s\n", buf);
        printf("%s\n", buf);
    } else {
        parseErrorCode(StatusCode);
    }

    LOG_FILE("eapiGetString() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiGetString() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eapiGetString() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eapiGetString() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
    
}

int eapiGetValue(EApiId_t id) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    uint32_t value = 0;
    
    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiGetValue() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eapiGetValue() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eapiGetValue() eApiLibInitialize return success\n");
    }

    StatusCode = EApiBoardGetValue(id, &value);
    if(StatusCode == EAPI_STATUS_SUCCESS) {
        if(id == EAPI_ID_GET_EAPI_SPEC_VERSION) {
            uint32_t ver = EAPI_VER_GET_VER(value);
            uint32_t rev = EAPI_VER_GET_REV(value);
            uint32_t build = EAPI_VER_GET_BUILD(value);
            LOG_FILE("Spec. Version: %d.%d.%d\n", ver, rev, build);
            printf("Spec. Version: %d.%d.%d\n", ver, rev, build);
        } else if(id == EAPI_ID_BOARD_PNPID_VAL) {
            uint16_t ascii_part = value; //a741
            uint8_t high_byte = (uint8_t)(ascii_part & 0x00FF);
            uint8_t low_byte = (uint8_t)(ascii_part >> 8);
            
            uint16_t lowtwo = ((uint16_t)high_byte << 8) | (uint16_t)low_byte;
            uint16_t secondB = lowtwo & 0x03E0;//0x1FE0;
            secondB = secondB >> 5;
            secondB = secondB - 1 + 65;

            uint16_t firstB = lowtwo & 0x7c00;//0xfc00;
            firstB = firstB >> 10;
            firstB = firstB - 1 + 65;

            uint16_t thirdB = lowtwo & 0x001F;
            thirdB = thirdB >> 0;
            thirdB = thirdB - 1 + 65;
            LOG_FILE("Board PNPID: %c%c%c\n", firstB, secondB, thirdB);
            printf("Board PNPID: %c%c%c\n", firstB, secondB, thirdB);
        } else if(id == EAPI_ID_BOARD_PLATFORM_REV_VAL) {
            uint32_t ver = EAPI_VER_GET_VER(value);
            uint32_t rev = EAPI_VER_GET_REV(value);
            uint32_t build = EAPI_VER_GET_BUILD(value);
            LOG_FILE("Platform Revision: %d.%d.%d\n", ver, rev, build);
            printf("Platform Revision: %d.%d.%d\n", ver, rev, build);
        } else if(id == EAPI_ID_BOARD_DRIVER_VERSION_VAL) {
            uint32_t ver = EAPI_VER_GET_VER(value);
            uint32_t rev = EAPI_VER_GET_REV(value);
            uint32_t build = EAPI_VER_GET_BUILD(value);
            LOG_FILE("Board Driver Version: %d.%d.%d\n", ver, rev, build);
            printf("Board Driver Version: %d.%d.%d\n", ver, rev, build);
        } else if(id == EAPI_ID_BOARD_LIB_VERSION_VAL) {
            uint32_t ver = EAPI_VER_GET_VER(value);
            uint32_t rev = EAPI_VER_GET_REV(value);
            uint32_t build = EAPI_VER_GET_BUILD(value);
            LOG_FILE("Board Lib. Version: %d.%d.%d\n", ver, rev, build);
            printf("Board Lib. Version: %d.%d.%d\n", ver, rev, build);
        } else if(id == EAPI_ID_HWMON_CPU_TEMP) {
            float tmpFloat = ((float)(value-EAPI_KELVINS_OFFSET)) / (float)10.0;
            LOG_FILE("CPU Temp.: %.2f Celcius (%d)\n", tmpFloat, value);
            printf("CPU Temp.: %.2f Celcius (%d)\n", tmpFloat, value);
        } else if(id == EAPI_ID_HWMON_CHIPSET_TEMP) {
            float tmpFloat = ((float)(value-EAPI_KELVINS_OFFSET)) / (float)10.0;
            LOG_FILE("Chipset Temp.: %.2f Celcius (%d)\n", tmpFloat, value);
            printf("Chipset Temp.: %.2f Celcius (%d)\n", tmpFloat, value);
        } else if(id == EAPI_ID_HWMON_SYSTEM_TEMP) {
            float tmpFloat = ((float)(value-EAPI_KELVINS_OFFSET)) / (float)10.0;
            LOG_FILE("System Temp.: %.2f Celcius (%d)\n", tmpFloat, value);
            printf("System Temp.: %.2f Celcius (%d)\n", tmpFloat, value);
        } else if(id == EAPI_ID_HWMON_VOLTAGE_VCORE) {
            LOG_FILE("VOLTAGE_VCORE: %d millivolts\n", value);
            printf("VOLTAGE_VCORE: %d millivolts\n", value);
        } else if(id == EAPI_ID_HWMON_VOLTAGE_2V5) {
            LOG_FILE("VOLTAGE_2V5: %d millivolts\n", value);
            printf("VOLTAGE_2V5: %d millivolts\n", value);
        } else if(id == EAPI_ID_HWMON_VOLTAGE_3V3) {
            LOG_FILE("VOLTAGE_3V3: %d millivolts\n", value);
            printf("VOLTAGE_3V3: %d millivolts\n", value);
        } else if(id == EAPI_ID_HWMON_VOLTAGE_VBAT) {
            LOG_FILE("VOLTAGE_VBAT: %d millivolts\n", value);
            printf("VOLTAGE_VBAT: %d millivolts\n", value);
        } else if(id == EAPI_ID_HWMON_VOLTAGE_5V) {
            LOG_FILE("VOLTAGE_5V: %d millivolts\n", value);
            printf("VOLTAGE_5V: %d millivolts\n", value);
        } else if(id == EAPI_ID_HWMON_VOLTAGE_5VSB) {
            LOG_FILE("VOLTAGE_5VSB: %d millivolts\n", value);
            printf("VOLTAGE_5VSB: %d millivolts\n", value);
        } else if(id == EAPI_ID_HWMON_VOLTAGE_12V) {
            LOG_FILE("VOLTAGE_12V: %d millivolts\n", value);
            printf("VOLTAGE_12V: %d millivolts\n", value);
        } else if(id == ASUS_ID_HWMON_VOLTAGE_VTT) {
            LOG_FILE("VOLTAGE_VTT: %d millivolts\n", value);
            printf("VOLTAGE_VTT: %d millivolts\n", value);
        } else if(id == ASUS_ID_HWMON_VOLTAGE_DCIN) {
            LOG_FILE("VOLTAGE_DCIN: %d millivolts\n", value);
            printf("VOLTAGE_DCIN: %d millivolts\n", value);
        } else if(id == EAPI_ID_HWMON_FAN_CPU) {
            LOG_FILE("FAN_CPU: %d RPM\n", value);
            printf("FAN_CPU: %d RPM\n", value);
        } else if(id == EAPI_ID_HWMON_FAN_SYSTEM) {
            LOG_FILE("FAN_SYSTEM: %d RPM\n", value);
            printf("FAN_SYSTEM: %d RPM\n", value);
        } else if(id == ASUS_ID_HWMON_CURRENT_SYSTEM) {
            LOG_FILE("SYSTEM_CURRENT %d milliamperes\n", value);
            printf("SYSTEM_CURRENT: %d milliamperes\n", value);
        } else {
            LOG_FILE("%d\n", value);
            printf("%d\n", value);
        }
    } else {
        parseErrorCode(StatusCode);
    }

    LOG_FILE("eapiGetValue() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiGetValue() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eapiGetValue() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eapiGetValue() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int eapiSetValue(EApiId_t id, uint32_t value) {
    EApiStatus_t StatusCode = ASUS_API_STATUS_ERROR;
    
    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiSetValue() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eapiSetValue() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eapiSetValue() eApiLibInitialize return success\n");
    }
    
    StatusCode = AsusBoardSetValue(id, value);
    if(StatusCode == ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("PASS\n");
        printf("PASS\n");
    } else {
        parseErrorCode(StatusCode);
    }

    LOG_FILE("eapiSetValue() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiSetValue() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eapiSetValue() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eapiSetValue() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int eapiWatchdog(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiWatchdog() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eapiWatchdog() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eapiWatchdog() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    if(strcmp(argv[2], "getcap") == 0) {
        uint32_t pMaxDelay = 0;
        uint32_t pMaxEventTimeout = 0;
        uint32_t pMaxResetTimeout = 0;
        LOG_FILE("eapiWatchdog() %s\n", argv[2]);
        StatusCode = EApiWDogGetCap(&pMaxDelay, &pMaxEventTimeout, &pMaxResetTimeout);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("MaxDelay:%d, MaxEventTimeout:%d, MaxResetTimeout:%d\n", pMaxDelay, pMaxEventTimeout, pMaxResetTimeout);
            printf("MaxDelay:%d, MaxEventTimeout:%d, MaxResetTimeout:%d\n", pMaxDelay, pMaxEventTimeout, pMaxResetTimeout);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "start") == 0) {
        uint32_t pMaxDelay = atoi(argv[3]);
        uint32_t pMaxEventTimeout = atoi(argv[4]);
        uint32_t pMaxResetTimeout = atoi(argv[5]);
        uint32_t triggerTime = atoi(argv[6]) / 1000; // change msec time to second
        LOG_FILE("eapiWatchdog() %s, delay:%d, eventTimeout:%d, resetTimeout:%d\n", argv[2], pMaxDelay, pMaxEventTimeout, pMaxResetTimeout);
        StatusCode = EApiWDogStart(pMaxDelay, pMaxEventTimeout, pMaxResetTimeout);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("EApiWDogStart() PASS\n");
            printf("EApiWDogStart() PASS\n");
        } else {
            LOGFILE_STDOUT ("EApiWDogStart() ERROR return:0x%02x\n", StatusCode);
            parseErrorCode(StatusCode);
            EApiStatus_t retStatusCode = EApiLibUnInitialize();
            LOG_FILE ("EApiLibUnInitialize return:%d\n", retStatusCode);
            return StatusCode;
        }

        int time = 0;
        while (time < 5) {
            StatusCode = EApiWDogTrigger();
            if(StatusCode == EAPI_STATUS_SUCCESS) {
                LOG_FILE("EApiWDogTrigger() PASS\n");
                printf("EApiWDogTrigger() PASS\n");
            } else {
                LOGFILE_STDOUT ("EApiWDogTrigger() ERROR return:0x%02x\n", StatusCode);
                parseErrorCode(StatusCode);
                EApiWDogStop();
                EApiStatus_t retStatusCode = EApiLibUnInitialize();
                LOG_FILE ("EApiLibUnInitialize return:%d\n", retStatusCode);
                return StatusCode;
            }
            sleep(triggerTime);
            time++;
        }
        StatusCode = EApiWDogStop();
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("EApiWDogStop() PASS\n");
            printf("EApiWDogStop() PASS\n");
        } else {
            LOGFILE_STDOUT ("EApiWDogStop ERROR return:0x%02x\n", StatusCode);
            parseErrorCode(StatusCode);
        }
    }

    LOG_FILE("EApiWDogStop() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("EApiWDogStop() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("EApiWDogStop() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("EApiWDogStop() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int eapiGpio(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    EApiId_t id = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiGpio() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eapiGpio() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eapiGpio() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    if(strcmp(argv[2], "getcap") == 0) {
        id = atoi(argv[3]);
        uint32_t pInputs = 0;
        uint32_t pOutputs = 0;
        LOG_FILE("eapiGpio() %s, id:%d\n", argv[2], id);
        StatusCode = EApiGPIOGetDirectionCaps(id, &pInputs, &pOutputs);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("id:%d, pInputs:%d, pOutputs:%d\n", id, pInputs, pOutputs);
            printf("GpioId:%d\n", id);
            printf("Input:0x%08x\n", pInputs);
            printf("Output:0x%08x\n", pOutputs);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getdirection") == 0) {
        id = atoi(argv[3]);
        uint32_t Bitmask = (uint32_t)strtol(argv[4], NULL, 0);
        uint32_t pDirection = 0;
        LOG_FILE("eapiGpio() %s, id:%d, bitmask:0x%02x\n", argv[2], id, Bitmask);
        StatusCode = EApiGPIOGetDirection(id, Bitmask, &pDirection);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            if(id == EAPI_GPIO_BANK_ID(0)) {
                LOG_FILE("BANK0:0x%08x\n", pDirection);
                printf("BANK0:0x%08x\n", pDirection);
            } else if(id == EAPI_GPIO_BANK_ID(32)) {
                LOG_FILE("BANK1:0x%08x\n", pDirection);
                printf("BANK1:0x%08x\n", pDirection);
            } else if(id == EAPI_GPIO_BANK_ID(64)) {
                LOG_FILE("BANK2:0x%08x\n", pDirection);
                printf("BANK2:0x%08x\n", pDirection);
            } else {
                if (pDirection == 1) {
                    LOG_FILE("GPIO%d:INPUT\n", id);
                    printf("GPIO%d:INPUT\n", id);
                } else if(pDirection == 0) {
                    LOG_FILE("GPIO%d:OUTPUT\n", id);
                    printf("GPIO%d:OUTPUT\n", id);
                } else {
                    LOG_FILE("GPIO%d:FAIL with direction:0x%08x\n", id, pDirection);
                    printf("GPIO%d:FAIL with direction:0x%08x\n", id, pDirection);
                }
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getlevel") == 0) {
        id = atoi(argv[3]);
        uint32_t Bitmask = (uint32_t)strtol(argv[4], NULL, 0);
        uint32_t pLevel = 0;
        LOG_FILE("eapiGpio() %s, id:%d, bitmask:0x%02x\n", argv[2], id, Bitmask);
        StatusCode = EApiGPIOGetLevel(id, Bitmask, &pLevel);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            if(id == EAPI_GPIO_BANK_ID(0)) {
                LOG_FILE("BANK0:0x%08x\n", pLevel);
                printf("BANK0:0x%08x\n", pLevel);
            } else if(id == EAPI_GPIO_BANK_ID(32)) {
                LOG_FILE("BANK1:0x%08x\n", pLevel);
                printf("BANK1:0x%08x\n", pLevel);
            } else if(id == EAPI_GPIO_BANK_ID(64)) {
                LOG_FILE("BANK2:0x%08x\n", pLevel);
                printf("BANK2:0x%08x\n", pLevel);
            } else {
                if(pLevel == 1) {
                    LOG_FILE("GPIO%d:HIGH\n", id);
                    printf("GPIO%d:HIGH\n", id);
                } else if(pLevel == 0) {
                    LOG_FILE("GPIO%d:LOW\n", id);
                    printf("GPIO%d:LOW\n", id);
                } else {
                    LOG_FILE("GPIO%d:FAIL with level:0x%08x\n", id, pLevel);
                    printf("GPIO%d:FAIL with level:0x%08x\n", id, pLevel);
                }
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setlevel") == 0) {
        id = atoi(argv[3]);
        uint32_t Bitmask = (uint32_t)strtol(argv[4], NULL, 0);
        uint32_t Level = (uint32_t)strtol(argv[5], NULL, 0);
        LOG_FILE("eapiGpio() %s, id:%d, bitmask:0x%02x, level:0x%02x\n", argv[2], id, Bitmask, Level);
        StatusCode = EApiGPIOSetLevel(id, Bitmask, Level);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setdirection") == 0) {
        id = atoi(argv[3]);
        uint32_t Bitmask = (uint32_t)strtol(argv[4], NULL, 0);
        uint32_t Direction = (uint32_t)strtol(argv[5], NULL, 0);
        LOG_FILE("eapiGpio() %s, id:%d, bitmask:0x%02x, direction:0x%02x\n", argv[2], id, Bitmask, Direction);
        StatusCode = EApiGPIOSetDirection(id, Bitmask, Direction);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("eapiGpio() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiGpio() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eapiGpio() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eapiGpio() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

void parseSystemBoot(uint32_t Frequency, uint32_t DayofMonth, uint32_t DayofWeek, uint32_t Hour, uint32_t Minute, uint32_t Second) {
    char* cFreg;
    if(Frequency == ASUS_SYSTEM_BOOT_DISABLE_FREQUENCY) {
        cFreg = "DISABLE";
    } else if(Frequency == ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_DAILY) {
        cFreg = "DAILY";
    } else if(Frequency == ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_WEEKLY) {
        cFreg = "WEEKLY";
    } else if(Frequency == ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_MONTHLY) {
        cFreg = "MONTHLY";
    } else {
        cFreg = "ERROR";
    }
    
    char* cDayofWeek;
    if(DayofWeek == ASUS_SYSTEM_BOOT_WEEKLY_SUNDAY) {
        cDayofWeek = "SUNDAY";
    } else if(DayofWeek == ASUS_SYSTEM_BOOT_WEEKLY_MONDAY) {
        cDayofWeek = "MONDAY";
    } else if(DayofWeek == ASUS_SYSTEM_BOOT_WEEKLY_TUESDAY) {
        cDayofWeek = "TUESDAY";
    } else if(DayofWeek == ASUS_SYSTEM_BOOT_WEEKLY_WEDNESDAY) {
        cDayofWeek = "WEDNESDAY";
    } else if(DayofWeek == ASUS_SYSTEM_BOOT_WEEKLY_THURSDAY) {
        cDayofWeek = "THURSDAY";
    } else if(DayofWeek == ASUS_SYSTEM_BOOT_WEEKLY_FRIDAY) {
        cDayofWeek = "FRIDAY";
    } else if(DayofWeek == ASUS_SYSTEM_BOOT_WEEKLY_SATURDAY) {
        cDayofWeek = "SATURDAY";
    } else {
        cDayofWeek = "ERROR";
    }
    LOG_FILE("Frequency:%s, DayofMonth:%d, DayofWeek:%s, Hour:%d, Minute:%d, Second:%d\n", cFreg, DayofMonth, cDayofWeek, Hour, Minute, Second);
    printf("Frequency:%s, DayofMonth:%d, DayofWeek:%s, Hour:%d, Minute:%d, Second:%d\n", cFreg, DayofMonth, cDayofWeek, Hour, Minute, Second);
}

int eapiLed(int argc, char *argv[]) {
    EApiStatus_t StatusCode = ASUS_API_STATUS_ERROR;
    uint32_t LedId = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("EapiLed() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("EapiLed() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("EapiLed() eApiLibInitialize return success\n");
    }
    StatusCode = ASUS_API_STATUS_ERROR;

    if(strcmp(argv[2], "turnoff") == 0) {
        LedId = atoi(argv[3]);
        StatusCode = AsusLedTurnOff(LedId);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setoccupied") == 0) {
        LedId = atoi(argv[3]);
        uint32_t occupied = atoi(argv[4]);
        StatusCode = AsusLedSetSystemOccupied(LedId, occupied);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "turnon") == 0) {
        LedId = atoi(argv[3]);
        uint32_t Color = atoi(argv[4]);
        StatusCode = AsusLedTurnOn(LedId, Color);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getnumberofleds") == 0) {
        uint32_t pLedCnt = 20;
        StatusCode = AsusLedGetNumberofLeds(&pLedCnt);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            printf("Number of Leds: %d\n", pLedCnt);
            LOG_FILE("Number of Leds: %d\n", pLedCnt);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getcap") == 0) {
        uint32_t pLedCnt = 20;
        StatusCode = AsusLedGetNumberofLeds(&pLedCnt);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(pLedCnt > 0) {
                struct LedInfo_s ledMap[pLedCnt];
                StatusCode = AsusLedGetInfo(ledMap, &pLedCnt);
                
                if(StatusCode == ASUS_API_STATUS_SUCCESS) {
                    LOG_FILE("Number of Leds: %d\n", pLedCnt);
                    for(int i=0; i<pLedCnt; i++) {
                        LOG_FILE("ledMap[%d].LedId: %d\n", i, ledMap[i].LedId);
                        printf("ledMap[%d].LedId: %d\n", i, ledMap[i].LedId);
                        LOG_FILE("ledMap[%d].SupportedColor: %d\n", i, ledMap[i].SupportedColor);
                        printf("ledMap[%d].SupportedColor: %d\n", i, ledMap[i].SupportedColor);
                        LOG_FILE("ledMap[%d].SystemOccupied: %d\n", i, ledMap[i].SystemOccupied);
                        printf("ledMap[%d].SystemOccupied: %d\n", i, ledMap[i].SystemOccupied);
                    }
                } else {
                    LOG_FILE("Number of Leds: %d\n", pLedCnt);
                    printf("Number of Leds: %d\n", pLedCnt);
                    parseErrorCode(StatusCode);
                }
            } else {
                LOG_FILE("ERROR, Number of Leds: %d\n", pLedCnt);
                printf("ERROR, Number of Leds: %d\n", pLedCnt);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("EapiLed() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("EapiLed() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("EapiLed() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("EapiLed() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int eapiSystemBoot(int argc, char *argv[]) {
    EApiStatus_t StatusCode = ASUS_API_STATUS_ERROR;
    EApiId_t id = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("EapiSystemBoot() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("EapiSystemBoot() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("EapiSystemBoot() eApiLibInitialize return success\n");
    }
    StatusCode = ASUS_API_STATUS_ERROR;

    if(strcmp(argv[2], "get") == 0) {
        id = atoi(argv[3]);
        uint32_t Frequency = 0;
        uint32_t DayofMonth = 0;
        uint32_t DayofWeek = 0;
        uint32_t Hour = 0;
        uint32_t Minute = 0;
        uint32_t Second = 0;
        StatusCode = AsusSystemBootGet(id, &Frequency, &DayofMonth, &DayofWeek, &Hour, &Minute, &Second);
        char* cId;
        if(id == ASUS_ID_SYSTEM_BOOT_BOOTUP) {
            cId = "BOOTUP";
        } else if(id == ASUS_ID_SYSTEM_BOOT_SHUTDOWN) {
            cId = "SHUTDOWN";
        } else if(id == ASUS_ID_SYSTEM_BOOT_SLEEP) {
            cId = "SLEEP";
        } else if(id == ASUS_ID_SYSTEM_BOOT_RESTART) {
            cId = "RESTART";
        } else {
            cId = "ERROR";
        }
        LOG_FILE("Id: %s\n", cId);
        printf("Id: %s\n", cId);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            parseSystemBoot(Frequency, DayofMonth, DayofWeek, Hour, Minute, Second);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "set") == 0) {
        id = atoi(argv[3]);
        uint32_t Frequency = atoi(argv[4]);
        uint32_t DayofMonth = atoi(argv[5]);
        uint32_t DayofWeek = atoi(argv[6]);
        uint32_t Hour = atoi(argv[7]);
        uint32_t Minute = atoi(argv[8]);
        uint32_t Second = atoi(argv[9]);
        StatusCode = AsusSystemBootSet(id, Frequency, DayofMonth, DayofWeek, Hour, Minute, Second);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("EapiSystemBoot() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("EapiSystemBoot() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("EapiSystemBoot() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("EapiSystemBoot() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int EApiVcpRtc(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    EApiId_t id = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpRtc() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eApiVcpRtc() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eApiVcpRtc() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    if(strcmp(argv[2], "get") == 0) {
        uint32_t Year;
        uint32_t Month;
        uint32_t DayofMonth;
        uint32_t Hour;
        uint32_t Minute;
        uint32_t Second;
        uint32_t DayofWeek;
        StatusCode = AsusVcpRtcTimeGet(&Year, &Month, &DayofMonth, &Hour, &Minute, &Second, &DayofWeek);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("Year:%d, Month:%d, DayofMonth:%d, Hour:%d, Minute:%d, Second:%d, DayofWeek:%d\n", Year, Month, DayofMonth, Hour, Minute, Second, DayofWeek);
            printf("Year:%d, Month:%d, DayofMonth:%d, Hour:%d, Minute:%d, Second:%d, DayofWeek:%d\n", Year, Month, DayofMonth, Hour, Minute, Second, DayofWeek);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "set") == 0) {
        uint32_t Year = atoi(argv[3]);
        uint32_t Month = atoi(argv[4]);
        uint32_t DayofMonth = atoi(argv[5]);
        uint32_t Hour = atoi(argv[6]);
        uint32_t Minute = atoi(argv[7]);
        uint32_t Second = atoi(argv[8]);
        uint32_t DayofWeek = atoi(argv[9]);
        StatusCode = AsusVcpRtcTimeSet(Year, Month, DayofMonth, Hour, Minute, Second, DayofWeek);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("eApiVcpRtc() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpRtc() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eApiVcpRtc() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eApiVcpRtc() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int EApiVcpPowerMgmt(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpPowerMgmt() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eApiVcpPowerMgmt() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE("eApiVcpPowerMgmt() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    if(strcmp(argv[2], "setlbpthreshold") == 0) {
        float Voltage = atof(argv[3]);
        LOG_FILE("eApiVcpPowerMgmt() %s, Voltage:%f\n", argv[2], Voltage);
        StatusCode = AsusVcpPowerMgmtSetLowBatteryProtectionThreshold(Voltage);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getlbpthreshold") == 0) {
        float Voltage;
        StatusCode = AsusVcpPowerMgmtGetLowBatteryProtectionThreshold(&Voltage);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("Voltage:%f\n", Voltage);
            printf("Voltage:%f\n", Voltage);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "seteventdelay") == 0) {
        uint32_t EventType = (uint32_t)strtol(argv[3], NULL, 0);
        uint32_t Second = (uint32_t)strtol(argv[4], NULL, 0);
        StatusCode = AsusVcpPowerMgmtSetEventDelay(EventType, Second);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "geteventdelay") == 0) {
        uint32_t EventType = (uint32_t)strtol(argv[3], NULL, 0);
        uint32_t pSecond;
        StatusCode = AsusVcpPowerMgmtGetEventDelay(EventType, &pSecond);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("EventType:0x%02x\n", EventType);
            printf("Seconds:%d\n", pSecond);
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("eApiVcpPowerMgmt() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpPowerMgmt() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eApiVcpPowerMgmt() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eApiVcpPowerMgmt() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int EApiVcpWakeup(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpWakeup() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eApiVcpWakeup() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eApiVcpWakeup() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    if(strcmp(argv[2], "enable") == 0) {
        uint32_t WakeupType = (uint32_t)strtol(argv[3], NULL, 0);
        StatusCode = AsusVcpWakeupTypeEnable(WakeupType);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "disable") == 0) {
        uint32_t WakeupType = (uint32_t)strtol(argv[3], NULL, 0);
        StatusCode = AsusVcpWakeupTypeDisable(WakeupType);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getenabledwakeuptype") == 0) {
        uint32_t pWakeupType;
        StatusCode = AsusVcpGetEnabledWakeupTypes(&pWakeupType);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("WakeupType:0x%02x\n", pWakeupType);
            printf("WakeupType:0x%02x\n", pWakeupType);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getlastwakeupsource") == 0) {
        uint32_t pWakeupType;
        StatusCode = AsusVcpGetLastWakeupSource(&pWakeupType);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOG_FILE("WakeupSource:0x%02x\n", pWakeupType);
            printf("WakeupSource:0x%02x\n", pWakeupType);
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("eApiVcpWakeup() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpWakeup() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eApiVcpWakeup() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eApiVcpWakeup() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int EApiVcpComPort(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    EApiId_t id = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpComPort() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eApiVcpComPort() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eApiVcpComPort() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    if(strcmp(argv[2], "enable") == 0) {
        uint32_t id = atoi(argv[3]);
        StatusCode = AsusVcpComPortEnable(id);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "disable") == 0) {
        uint32_t id = atoi(argv[3]);
        StatusCode = AsusVcpComPortDisable(id);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "read") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s \n", argv[2], argv[3], argv[4]);
        uint32_t Id = atoi(argv[3]);
        
        char pBuffer[1024];
        uint32_t BufLen = 1024;
        uint32_t ByteCnt = (uint32_t)strtol(argv[4], NULL, 0);
        memset(pBuffer, 0, BufLen);
        StatusCode = AsusVcpComPortRead(Id, pBuffer, BufLen, ByteCnt);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            for(int i=0; i<ByteCnt; i++) {
                LOGFILE_STDOUT("pBuffer[%d]:0x%02x\n", i, pBuffer[i]);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "write") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s, argv[5]:%s \n", argv[2], argv[3], argv[4], argv[5]);
        
        uint32_t Id = atoi(argv[3]);
        uint32_t ByteCnt = (uint32_t)strtol(argv[5], NULL, 0);
        
        char pBuffer[ByteCnt];
        memset(pBuffer, 0, ByteCnt);
        memcpy(pBuffer, argv[4], ByteCnt);

        
        StatusCode = AsusVcpComPortWrite(Id, pBuffer, ByteCnt);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOGFILE_STDOUT("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "init") == 0) {
        uint32_t Id = atoi(argv[3]);
        uint32_t Mode = atoi(argv[4]);
        uint32_t BaudRate = atoi(argv[5]);
        uint32_t Parity = atoi(argv[6]);
        uint32_t DataWidth = atoi(argv[7]);
        uint32_t StopBit = atoi(argv[8]);
        uint32_t FlowControl = atoi(argv[9]);
        StatusCode = AsusVcpComPortInit(Id, Mode, BaudRate, Parity, DataWidth, StopBit, FlowControl);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("eApiVcpComPort() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpComPort() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eApiVcpComPort() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eApiVcpComPort() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int EApiVcpGSensor(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    EApiId_t id = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpGSensor() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eApiVcpGSensor() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eApiVcpGSensor() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    if(strcmp(argv[2], "read") == 0) {
        VcpGSensorValue_t value;
        StatusCode = AsusVcpGSensorRead(&value);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("x: %d\n", value.x_mg);
            printf("x: %d\n", value.x_mg);
            LOG_FILE("y: %d\n", value.y_mg);
            printf("y: %d\n", value.y_mg);
            LOG_FILE("z: %d\n", value.z_mg);
            printf("z: %d\n", value.z_mg);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getoffset") == 0) {
        VcpGSensorValue_t value;
        StatusCode = AsusVcpGSensorGetOffset(&value);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("x: %d\n", value.x_mg);
            printf("x: %d\n", value.x_mg);
            LOG_FILE("y: %d\n", value.y_mg);
            printf("y: %d\n", value.y_mg);
            LOG_FILE("z: %d\n", value.z_mg);
            printf("z: %d\n", value.z_mg);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setoffset") == 0) {
        VcpGSensorValue_t mg_offset;
        mg_offset.x_mg = strtol(argv[2], NULL, 10);
        mg_offset.y_mg = strtol(argv[3], NULL, 10);
        mg_offset.z_mg = strtol(argv[4], NULL, 10);
        StatusCode = AsusVcpGSensorSetOffset(mg_offset);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "enable") == 0) {
        StatusCode = AsusVcpGSensorEnable();
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "disable") == 0) {
        StatusCode = AsusVcpGSensorDisable();
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getresolution") == 0) {
        VCP_GSENSOR_RES res;
        StatusCode = AsusVcpGSensorGetResolution(&res);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(res == VCP_GSENSOR_RES_2G) {
                LOG_FILE("2G\n");
                printf("2G\n");
            } else if(res == VCP_GSENSOR_RES_4G) {
                LOG_FILE("4G\n");
                printf("4G\n");
            } else if(res == VCP_GSENSOR_RES_8G) {
                LOG_FILE("8G\n");
                printf("8G\n");
            } else if(res == VCP_GSENSOR_RES_16G) {
                LOG_FILE("16G\n");
                printf("16G\n");
            } else {
                printf("FAIL, code:%d\n", StatusCode);
                LOG_FILE("FAIL, code:%d\n", StatusCode);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setalarmthreshold") == 0) {
        char *argFirst;
        VCP_GSENSOR_RES res;
        argFirst = argv[3];
        if(strcmp(argFirst, "2g") == 0) {
            res = VCP_GSENSOR_RES_2G;
        } else if(strcmp(argFirst, "4g") == 0) {
            res = VCP_GSENSOR_RES_4G;
        } else if(strcmp(argFirst, "8g") == 0) {
            res = VCP_GSENSOR_RES_8G;
        } else if(strcmp(argFirst, "16g") == 0) {
            res = VCP_GSENSOR_RES_16G;
        }
        int mg = strtol(argv[4], NULL, 10);
        StatusCode = AsusVcpGSensorSetAlarmThreshold(res, mg);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getalarmthreshold") == 0) {
        VCP_GSENSOR_RES res;
        int value;
        StatusCode = AsusVcpGSensorGetAlarmThreshold(&res, &value);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(res == VCP_GSENSOR_RES_2G) {
                LOG_FILE("%d, 2G\n", value);
                printf("%d, 2G\n", value);
            } else if(res == VCP_GSENSOR_RES_4G) {
                LOG_FILE("%d, 4G\n", value);
                printf("%d, 4G\n", value);
            } else if(res == VCP_GSENSOR_RES_8G) {
                LOG_FILE("%d, 8G\n", value);
                printf("%d, 8G\n", value);
            } else if(res == VCP_GSENSOR_RES_16G) {
                LOG_FILE("%d, 16G\n", value);
                printf("%d, 16G\n", value);
            } else {
                printf("FAIL, code:%d\n", StatusCode);
                LOG_FILE("FAIL, code:%d\n", StatusCode);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "disablealarm") == 0) {
        VCP_VCP_GSENSOR_ALARM_ENABLE_SETTING value = VCP_GSENSOR_ALARM_DISABLE;
        StatusCode = AsusVcpGSensorSetAlarm(value);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {            
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "enablealarm") == 0) {
        char *argFirst;
        VCP_VCP_GSENSOR_ALARM_ENABLE_SETTING value;
        argFirst = argv[3];
        if(strcmp(argFirst, "x") == 0) {
            value = VCP_GSENSOR_ALARM_ENABLE_X;
        } else if(strcmp(argFirst, "y") == 0) {
            value = VCP_GSENSOR_ALARM_ENABLE_Y;
        } else if(strcmp(argFirst, "z") == 0) {
            value = VCP_GSENSOR_ALARM_ENABLE_Z;
        } else if(strcmp(argFirst, "xz") == 0) {
            value = VCP_GSENSOR_ALARM_ENABLE_X_Z;
        } else if(strcmp(argFirst, "yz") == 0) {
            value = VCP_GSENSOR_ALARM_ENABLE_Y_Z;
        } else if(strcmp(argFirst, "xy") == 0) {
            value = VCP_GSENSOR_ALARM_ENABLE_X_Y;
        } else if(strcmp(argFirst, "xyz") == 0) {
            value = VCP_GSENSOR_ALARM_ENABLE_X_Y_Z;
        }
        
        StatusCode = AsusVcpGSensorSetAlarm(value);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getalarm") == 0) {
        VCP_VCP_GSENSOR_ALARM_ENABLE_SETTING value;
        StatusCode = AsusVcpGSensorGetAlarm(&value);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(value == VCP_GSENSOR_ALARM_DISABLE) {
                LOG_FILE("DISABLE\n");
                printf("DISABLE\n");
            } else if(value == VCP_GSENSOR_ALARM_ENABLE_X) {
                LOG_FILE("ENABLE_X\n");
                printf("ENABLE_X\n");
            } else if(value == VCP_GSENSOR_ALARM_ENABLE_Y) {
                LOG_FILE("ENABLE_Y\n");
                printf("ENABLE_Y\n");
            } else if(value == VCP_GSENSOR_ALARM_ENABLE_Z) {
                LOG_FILE("ENABLE_Z\n");
                printf("ENABLE_Z\n");
            } else if(value == VCP_GSENSOR_ALARM_ENABLE_X_Z) {
                LOG_FILE("ENABLE_X_Z\n");
                printf("ENABLE_X_Z\n");
            } else if(value == VCP_GSENSOR_ALARM_ENABLE_Y_Z) {
                LOG_FILE("ENABLE_Y_Z\n");
                printf("ENABLE_Y_Z\n");
            } else if(value == VCP_GSENSOR_ALARM_ENABLE_X_Y) {
                LOG_FILE("ENABLE_X_Y\n");
                printf("ENABLE_X_Y\n");
            } else if(value == VCP_GSENSOR_ALARM_ENABLE_X_Y_Z) {
                LOG_FILE("ENABLE_X_Y_Z\n");
                printf("ENABLE_X_Y_Z\n");
            } else {
                printf("FAIL, code:%d\n", StatusCode);
                LOG_FILE("FAIL, code:%d\n", StatusCode);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "available") == 0) {
        char supported;
        StatusCode = AsusVcpGSensorAvailable(&supported);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(supported == 0) {
                LOG_FILE("0\n");
                printf("0\n");
            } else if(supported == 1) {
                LOG_FILE("1\n");
                printf("1\n");
            } else {
                LOG_FILE("FAIL StatusCode:%d \n", StatusCode);
                printf("FAIL StatusCode:%d \n", StatusCode);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getstatus") == 0) {
        char status;
        StatusCode = AsusVcpGSensorGetStatus(&status);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(status == 0) {
                LOG_FILE("0\n");
                printf("0\n");
            } else if(status == 1) {
                LOG_FILE("1\n");
                printf("1\n");
            } else {
                LOG_FILE("FAIL StatusCode:%d \n", StatusCode);
                printf("FAIL StatusCode:%d \n", StatusCode);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setresolution") == 0) {
        char *argFirst;
        VCP_GSENSOR_RES res;
        argFirst = argv[3];
        if(strcmp(argFirst, "2g") == 0) {
            res = VCP_GSENSOR_RES_2G;
        } else if(strcmp(argFirst, "4g") == 0) {
            res = VCP_GSENSOR_RES_4G;
        } else if(strcmp(argFirst, "8g") == 0) {
            res = VCP_GSENSOR_RES_8G;
        } else if(strcmp(argFirst, "16g") == 0) {
            res = VCP_GSENSOR_RES_16G;
        }
        
        StatusCode = AsusVcpGSensorSetResolution(res);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("eApiVcpGSensor() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eApiVcpGSensor() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eApiVcpGSensor() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eApiVcpGSensor() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int eapiI2C(int argc, char *argv[]) {
    EApiStatus_t StatusCode = EAPI_STATUS_ERROR;
    EApiId_t id = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiI2C() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eapiI2C() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eapiI2C() eApiLibInitialize return success\n");
    }
    StatusCode = EAPI_STATUS_ERROR;

    //getbuscap <I2C Bus Id>
    if(strcmp(argv[2], "getbuscap") == 0) {
        //id = atoi(argv[3]);
        id = (int)strtol(argv[3], NULL, 0);
        uint32_t pMaxBlkLen = 0;
        StatusCode = EApiI2CGetBusCap(id, &pMaxBlkLen);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOGFILE_STDOUT ("pMaxBlkLen:%d\n", pMaxBlkLen);
        } else {
            parseErrorCode(StatusCode);
        }
    //probedevice <I2C Bus Id> <Encoded 7Bit I2C Device Address>
    } else if(strcmp(argv[2], "probedevice") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s\n", argv[2], argv[3], argv[4]);
        //id = atoi(argv[3]);
        //uint32_t Addr = atoi(argv[4]);
        id = (int)strtol(argv[3], NULL, 0);
        uint32_t Addr = (uint32_t)strtol(argv[4], NULL, 0);
        Addr = EAPI_I2C_ENC_7BIT_ADDR(Addr);
        StatusCode = EApiI2CProbeDevice(id, Addr);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOGFILE_STDOUT("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    //readtransfer <I2C Bus Id> <Encoded 7Bit I2C Device Address> <I2C Command/Offset> <Byte Count to read>
    } else if(strcmp(argv[2], "readtransfer") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s, argv[5]:%s, argv[6]:%s\n", argv[2], argv[3], argv[4], argv[5], argv[6]);
        //id = atoi(argv[3]);
        //uint32_t Addr = atoi(argv[4]);
        id = (int)strtol(argv[3], NULL, 0);
        uint32_t Addr = (uint32_t)strtol(argv[4], NULL, 0);
        Addr = EAPI_I2C_ENC_7BIT_ADDR(Addr);
        //uint32_t Cmd = atoi(argv[5]);
        uint32_t Cmd = (uint32_t)strtol(argv[5], NULL, 0);
        char pBuffer[1024];
        uint32_t BufLen = 1024;
        uint32_t ByteCnt = (uint32_t)strtol(argv[6], NULL, 0);
        memset(pBuffer, 0, BufLen);
        StatusCode = EApiI2CReadTransfer(id, Addr, Cmd, pBuffer, BufLen, ByteCnt);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            for(int i=0; i<ByteCnt; i++) {
                LOGFILE_STDOUT("pBuffer[%d]:0x%02x\n", i, pBuffer[i]);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    //writetransfer <I2C Bus Id> <Encoded 7Bit I2C Device Address> <I2C Command/Offset> <Write Data> <Byte Count to write>
    } else if(strcmp(argv[2], "writetransfer") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s, argv[5]:%s, argv[6]:%s, argv[7]:%s\n", argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
        //id = atoi(argv[3]);
        //uint32_t Addr = atoi(argv[4]);
        id = (int)strtol(argv[3], NULL, 0);
        uint32_t Addr = (uint32_t)strtol(argv[4], NULL, 0);
        Addr = EAPI_I2C_ENC_7BIT_ADDR(Addr);
        //uint32_t Cmd = atoi(argv[5]);
        uint32_t Cmd = (uint32_t)strtol(argv[5], NULL, 0);
        char pBuffer[1024];
        uint32_t BufLen = 1024;
        memset(pBuffer, 0, BufLen);
        uint32_t pWBuffer32 = (uint32_t)strtol(argv[6], NULL, 0);
        pBuffer[0]=(uint8_t)(pWBuffer32&0xFF);
            
        uint32_t ByteCnt = (uint32_t)strtol(argv[7], NULL, 0);
        StatusCode = EApiI2CWriteTransfer(id, Addr, Cmd, pBuffer, ByteCnt);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOGFILE_STDOUT("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    //writereadraw <I2C Bus Id> <Encoded 7Bit I2C Device Address> <Write Data> <Number of Bytes to write plus 1> <Number of Bytes to Read plus 1>
    } else if(strcmp(argv[2], "writereadraw") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s, argv[5]:%s, argv[6]:%s, argv[7]:%s\n", argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]);
        id = (int)strtol(argv[3], NULL, 0);
        uint32_t Addr = (uint32_t)strtol(argv[4], NULL, 0);
        //uint32_t pWBuffer32 = atoi(argv[5]);
        uint32_t pWBuffer32 = (uint32_t)strtol(argv[5], NULL, 0);
        uint32_t WriteBCnt = (uint32_t)strtol(argv[6], NULL, 0);
        Addr = EAPI_I2C_ENC_7BIT_ADDR(Addr);
        char pRBuffer[1024];
        uint32_t RBufLen = 1024;
        uint32_t ReadBCnt = (uint32_t)strtol(argv[7], NULL, 0);
        memset(pRBuffer, 0, RBufLen);
        if(WriteBCnt <= 0) {
            StatusCode = EApiI2CWriteReadRaw(id, Addr, NULL, 0, pRBuffer, RBufLen, ReadBCnt);
        } else {
            uint8_t LclpBuffer[8]={0};
            LclpBuffer[0]=(uint8_t)(pWBuffer32&0xFF);
            StatusCode = EApiI2CWriteReadRaw(id, Addr, LclpBuffer, WriteBCnt, pRBuffer, RBufLen, ReadBCnt);
        }
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            for(int i=0; i<ReadBCnt-1; i++) {
                LOGFILE_STDOUT("pRBuffer[%d]:0x%02x\n", i, pRBuffer[i]);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    //writeraw <I2C Bus Id> <Encoded 7Bit I2C Device Address> <Write Data> <Number of Bytes to write plus 1>
    } else if(strcmp(argv[2], "writeraw") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s, argv[5]:%s, argv[6]:%s\n", argv[2], argv[3], argv[4], argv[5], argv[6]);
        id = (int)strtol(argv[3], NULL, 0);
        uint32_t Addr = (uint32_t)strtol(argv[4], NULL, 0);
        uint32_t pWBuffer32 = (uint32_t)strtol(argv[5], NULL, 0);
        uint32_t WriteBCnt = (uint32_t)strtol(argv[6], NULL, 0);
        Addr = EAPI_I2C_ENC_7BIT_ADDR(Addr);

        uint8_t LclpBuffer[8]={0};
        LclpBuffer[0]=(uint8_t)(pWBuffer32&0xFF);
        StatusCode = EApiI2CWriteReadRaw(id, Addr, LclpBuffer, WriteBCnt, NULL, 0, 0);
        
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            LOGFILE_STDOUT("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    //readraw <I2C Bus Id> <Encoded 7Bit I2C Device Address> <Number of Bytes to Read plus 1>
    } else if(strcmp(argv[2], "readraw") == 0) {
        LOG_FILE("argv[2]:%s, argv[3]:%s, argv[4]:%s, argv[5]:%s\n", argv[2], argv[3], argv[4], argv[5]);
        id = (int)strtol(argv[3], NULL, 0);
        uint32_t Addr = (uint32_t)strtol(argv[4], NULL, 0);
        Addr = EAPI_I2C_ENC_7BIT_ADDR(Addr);
        char pRBuffer[1024];
        uint32_t RBufLen = 1024;
        uint32_t ReadBCnt = (uint32_t)strtol(argv[5], NULL, 0);
        memset(pRBuffer, 0, RBufLen);
        StatusCode = EApiI2CWriteReadRaw(id, Addr, NULL, 0, pRBuffer, RBufLen, ReadBCnt);
        if(StatusCode == EAPI_STATUS_SUCCESS) {
            for(int i=0; i<ReadBCnt-1; i++) {
                LOGFILE_STDOUT("pRBuffer[%d]:0x%02x\n", i, pRBuffer[i]);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    }
    LOG_FILE("eapiI2C() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiI2C() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eapiI2C() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eapiI2C() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int eapiModemMgmt(int argc, char *argv[]) {
    EApiStatus_t StatusCode = ASUS_API_STATUS_ERROR;
    EApiId_t id = 0;

    StatusCode = EApiLibInitialize();
    if(StatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiModemMgmt() FAIL on eApiLibInitialize:%d\n", StatusCode);
        printf("eapiModemMgmt() FAIL on eApiLibInitialize:%d\n", StatusCode);
        return StatusCode;
    } else {
        LOG_FILE ("eapiModemMgmt() eApiLibInitialize return success\n");
    }
    StatusCode = ASUS_API_STATUS_ERROR;
    if(strcmp(argv[2], "startnetwork") == 0) {
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemStartNetwork(index);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getmodeminfo") == 0) {
        uint32_t pModemCnt = 20;
        LOG_FILE("eapiModemMgmt() %s\n", argv[2]);
        StatusCode = AsusConnMgrModemGetNumberofModems(&pModemCnt);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(pModemCnt > 0) {
                struct ConnMgrModemInfo_s ModemMap[pModemCnt];
                StatusCode = AsusConnMgrModemGetModemInfo(ModemMap, &pModemCnt);
                
                if(StatusCode == ASUS_API_STATUS_SUCCESS) {
                    LOG_FILE("Number of Modems: %d\n", pModemCnt);
                    if(pModemCnt <= 0) {
                        printf("Number of Modems: %d\n", pModemCnt);
                    } else {
                        for(int i=0; i<pModemCnt; i++) {
                            LOG_FILE("{\"index\":%c,\"FirmwareRevision\":\"%s\",\"ModemName\":\"%s\",\"Manufacturer\":\"%s\",\"Path\":\"%s\"}\n", ModemMap[i].Index, ModemMap[i].FirmwareRevision, ModemMap[i].ModemName, ModemMap[i].Manufacturer, ModemMap[i].Path);
                            printf("{\"index\":%c,\"FirmwareRevision\":\"%s\",\"ModemName\":\"%s\",\"Manufacturer\":\"%s\",\"Path\":\"%s\"}\n", ModemMap[i].Index, ModemMap[i].FirmwareRevision, ModemMap[i].ModemName, ModemMap[i].Manufacturer, ModemMap[i].Path);
                        }
                    }
                } else {
                    LOG_FILE("Number of Modems: %d\n", pModemCnt);
                    printf("Number of Modems: %d\n", pModemCnt);
                    parseErrorCode(StatusCode);
                }
            } else {
                LOG_FILE("ERROR, Number of Modems: %d\n", pModemCnt);
                printf("ERROR, Number of Modems: %d\n", pModemCnt);
            }
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getnumberofmodems") == 0) {
        uint32_t pModemCnt = 20;
        LOG_FILE("eapiModemMgmt() %s\n", argv[2]);
        StatusCode = AsusConnMgrModemGetNumberofModems(&pModemCnt);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("Number of Modems: %d\n", pModemCnt);
            printf("Number of Modems: %d\n", pModemCnt);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "stopnetwork") == 0) {
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemStopNetwork(index);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "poweron") == 0) {
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemPowerOn(index);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "poweroff") == 0) {
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemPowerOff(index);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "restart") == 0) {
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemRestart(index);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "resetprofile") == 0) {
        LOG_FILE("eapiModemMgmt() %s\n", argv[2]);
        StatusCode = AsusConnMgrModemResetProfile();
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setfailover") == 0) {
        uint32_t enable = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, enable:%d\n", argv[2], enable);
        StatusCode = AsusConnMgrSetFailover(enable);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setkeepalive") == 0) {
        uint32_t enable = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, enable:%d\n", argv[2], enable);
        StatusCode = AsusConnMgrModemSetKeepAlive(enable);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "switchsim") == 0) {
        uint32_t slotid = (int)strtol(argv[4], NULL, 0);
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d, slotid:%d\n", argv[2], index, slotid);
        StatusCode = AsusConnMgrModemSwitchSIM(index, slotid);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getattachedstatus") == 0) {
        ConnMgrModemAttachedStatus_t status;
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemGetAttachedStatus(index, &status);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("RegistrationState:%d\n", status.RegistrationState);
            printf("RegistrationState:%d\n", status.RegistrationState);
            LOG_FILE("RadioInterface:%d\n", status.RadioInterface);
            printf("RadioInterface:%d\n", status.RadioInterface);
            LOG_FILE("FlightModeState:%d\n", status.FlightModeState);
            printf("FlightModeState:%d\n", status.FlightModeState);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getstatus") == 0) {
        
        char pConnected[1024];
        uint32_t pConnectedLen = 1024;
        memset(pConnected, 0, pConnectedLen);
        
        char pInterface[1024];
        uint32_t pInterfaceLen = 1024;
        memset(pInterface, 0, pInterfaceLen);
        
        char pApn[1024];
        uint32_t pApnLen = 1024;
        memset(pApn, 0, pApnLen);
        
        char pRoaming[1024];
        uint32_t pRoamingLen = 1024;
        memset(pRoaming, 0, pRoamingLen);

        char pAddress[1024];
        uint32_t pAddressLen = 1024;
        memset(pAddress, 0, pAddressLen);

        char pGateway[1024];
        uint32_t pGatewayLen = 1024;
        memset(pGateway, 0, pGatewayLen);

        char pMtu[1024];
        uint32_t pMtuLen = 1024;
        memset(pMtu, 0, pMtuLen);

        char pDns[1024];
        uint32_t pDnsLen = 1024;
        memset(pDns, 0, pDnsLen);

        ConnMgrModemIpv4Status_t p4Value = {
                pAddress,
                &pAddressLen,
                pGateway,
                &pGatewayLen,
                pMtu,
                &pMtuLen,
                pDns,
                &pDnsLen,
        };

        char pAddress6[1024];
        uint32_t pAddressLen6 = 1024;
        memset(pAddress6, 0, pAddressLen6);

        char pGateway6[1024];
        uint32_t pGatewayLen6 = 1024;
        memset(pGateway6, 0, pGatewayLen6);

        char pMtu6[1024];
        uint32_t pMtuLen6 = 1024;
        memset(pMtu6, 0, pMtuLen6);

        char pDns6[1024];
        uint32_t pDnsLen6 = 1024;
        memset(pDns6, 0, pDnsLen6);

        ConnMgrModemIpv6Status_t p6Value = {
                pAddress6,
                &pAddressLen6,
                pGateway6,
                &pGatewayLen6,
                pMtu6,
                &pMtuLen6,
                pDns6,
                &pDnsLen6,
        };

        ConnMgrModemStatus_t pValue = {
                pConnected,
                &pConnectedLen,
                pInterface,
                &pInterfaceLen,
                pApn,
                &pApnLen,
                pRoaming,
                &pRoamingLen,
                &p4Value,
                &p6Value
        };
        
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemGetStatus(index, &pValue);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("Connected:%s\n", pValue.pConnected);
            printf("Connected:%s\n", pValue.pConnected);
            LOG_FILE("%d\n", *(pValue.pConnectedLen));
            LOG_FILE("Interface:%s\n", pValue.pInterface);
            printf("Interface:%s\n", pValue.pInterface);
            LOG_FILE("%d\n", *(pValue.pInterfaceLen));
            LOG_FILE("Apn:%s\n", pValue.pApn);
            printf("Apn:%s\n", pValue.pApn);
            LOG_FILE("%d\n", *(pValue.pApnLen));
            LOG_FILE("Roaming:%s\n", pValue.pRoaming);
            printf("Roaming:%s\n", pValue.pRoaming);
            LOG_FILE("%d\n", *(pValue.pRoamingLen));

            LOG_FILE("IPv4_Address:%s\n", pValue.pIpv4Status->pAddress);
            printf("IPv4_Address:%s\n", pValue.pIpv4Status->pAddress);
            LOG_FILE("%d\n", *(pValue.pIpv4Status->pAddressLen));
            LOG_FILE("IPv4_Gateway:%s\n", pValue.pIpv4Status->pGateway);
            printf("IPv4_Gateway:%s\n", pValue.pIpv4Status->pGateway);
            LOG_FILE("%d\n", *(pValue.pIpv4Status->pGatewayLen));
            LOG_FILE("IPv4_Mtu:%s\n", pValue.pIpv4Status->pMtu);
            printf("IPv4_Mtu:%s\n", pValue.pIpv4Status->pMtu);
            LOG_FILE("%d\n", *(pValue.pIpv4Status->pMtuLen));
            LOG_FILE("IPv4_Dns:%s\n", pValue.pIpv4Status->pDns);
            printf("IPv4_Dns:%s\n", pValue.pIpv4Status->pDns);
            LOG_FILE("%d\n", *(pValue.pIpv4Status->pDnsLen));
            
            LOG_FILE("IPv6_Address:%s\n", pValue.pIpv6Status->pAddress);
            printf("IPv6_Address:%s\n", pValue.pIpv6Status->pAddress);
            LOG_FILE("%d\n", *(pValue.pIpv6Status->pAddressLen));
            LOG_FILE("IPv6_Gateway:%s\n", pValue.pIpv6Status->pGateway);
            printf("IPv6_Gateway:%s\n", pValue.pIpv6Status->pGateway);
            LOG_FILE("%d\n", *(pValue.pIpv6Status->pGatewayLen));
            LOG_FILE("IPv6_Mtu:%s\n", pValue.pIpv6Status->pMtu);
            printf("IPv6_Mtu:%s\n", pValue.pIpv6Status->pMtu);
            LOG_FILE("%d\n", *(pValue.pIpv6Status->pMtuLen));
            LOG_FILE("IPv6_Dns:%s\n", pValue.pIpv6Status->pDns);
            printf("IPv6_Dns:%s\n", pValue.pIpv6Status->pDns);
            LOG_FILE("%d\n", *(pValue.pIpv6Status->pDnsLen));
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "unlockpin") == 0) {
        char pBuffer[1024];
        uint32_t BufLen = 1024;
        memset(pBuffer, 0, BufLen);
        strncpy (pBuffer, argv[4], strlen(argv[4]));
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d, pin:%s\n", argv[2], index, pBuffer);
        StatusCode = AsusConnMgrModemUnlockSIMByPIN(index, pBuffer);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setflightmode") == 0) {
        uint32_t tmp = (int)strtol(argv[4], NULL, 0);
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        enum ASUS_CMM_FLIGHT_MODE_STATE flightMode = tmp;
        LOG_FILE("eapiModemMgmt() %s, index:%d, flightMode:%d\n", argv[2], index, flightMode);
        StatusCode = AsusConnMgrModemSetFlightMode(index, flightMode);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setapn") == 0) {
        char pBuffer[1024];
        uint32_t BufLen = 1024;
        memset(pBuffer, 0, BufLen);
        strncpy (pBuffer, argv[3], strlen(argv[3]));
        LOG_FILE("eapiModemMgmt() %s, apn:%s\n", argv[2], pBuffer);
        StatusCode = AsusConnMgrModemSetAPN(pBuffer);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setuser") == 0) {
        char pBuffer[1024];
        uint32_t BufLen = 1024;
        memset(pBuffer, 0, BufLen);
        strncpy (pBuffer, argv[3], strlen(argv[3]));
        LOG_FILE("eapiModemMgmt() %s, user:%s\n", argv[2], pBuffer);
        StatusCode = AsusConnMgrModemSetUser(pBuffer);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setpassword") == 0) {
        char pBuffer[1024];
        uint32_t BufLen = 1024;
        memset(pBuffer, 0, BufLen);
        strncpy (pBuffer, argv[3], strlen(argv[3]));
        LOG_FILE("eapiModemMgmt() %s, password:%s\n", argv[2], pBuffer);
        StatusCode = AsusConnMgrModemSetPassword(pBuffer);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setiptype") == 0) {
        uint32_t input = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, iptype:%d\n", argv[2], input);
        StatusCode = AsusConnMgrModemSetIPType(input);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getadvancedsignalinfo") == 0) {
        ConnMgrModemAdvSignalInfo_t pValue;
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemGetAdvancedSignalInfo(index, &pValue);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            printf("Evdo===============================================\n");
            LOG_FILE("Valid Flag: 0x%02x\n", pValue.Evdo.validFlag);
            printf("Valid Flag: 0x%02x\n", pValue.Evdo.validFlag);
            LOG_FILE("RSSI: %f\n", pValue.Evdo.rssi);
            printf("RSSI: %f\n", pValue.Evdo.rssi);
            LOG_FILE("ECIO: %f\n", pValue.Evdo.ecio);
            printf("ECIO: %f\n", pValue.Evdo.ecio);
            LOG_FILE("SINR: %f\n", pValue.Evdo.sinr);
            printf("SINR: %f\n", pValue.Evdo.sinr);
            LOG_FILE("IO: %f\n", pValue.Evdo.io);
            printf("IO: %f\n", pValue.Evdo.io);
            
            printf("GSM================================================\n");
            LOG_FILE("Valid Flag: 0x%02x\n", pValue.Gsm.validFlag);
            printf("Valid Flag: 0x%02x\n", pValue.Gsm.validFlag);
            LOG_FILE("RSSI: %f\n", pValue.Gsm.rssi);
            printf("RSSI: %f\n", pValue.Gsm.rssi);
            
            printf("UMTS===============================================\n");
            LOG_FILE("Valid Flag: 0x%02x\n", pValue.Umts.validFlag);
            printf("Valid Flag: 0x%02x\n", pValue.Umts.validFlag);
            LOG_FILE("RSSI: %f\n", pValue.Umts.rssi);
            printf("RSSI: %f\n", pValue.Umts.rssi);
            LOG_FILE("RSCP: %f\n", pValue.Umts.rscp);
            printf("RSCP: %f\n", pValue.Umts.rscp);
            LOG_FILE("ECIO: %f\n", pValue.Umts.ecio);
            printf("ECIO: %f\n", pValue.Umts.ecio);
            
            printf("LTE================================================\n");
            LOG_FILE("Valid Flag: 0x%02x\n", pValue.Lte.validFlag);
            printf("Valid Flag: 0x%02x\n", pValue.Lte.validFlag);
            LOG_FILE("RSSI: %f\n", pValue.Lte.rssi);
            printf("RSSI: %f\n", pValue.Lte.rssi);
            LOG_FILE("RSRQ: %f\n", pValue.Lte.rsrq);
            printf("RSRQ: %f\n", pValue.Lte.rsrq);
            LOG_FILE("RSRP: %f\n", pValue.Lte.rsrp);
            printf("RSRP: %f\n", pValue.Lte.rsrp);
            LOG_FILE("SNR: %f\n", pValue.Lte.snr);
            printf("SNR: %f\n", pValue.Lte.snr);
            
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "checkcarrier") == 0) {
        char pMcc[1024];
        uint32_t pMccLen = 1024;
        memset(pMcc, 0, pMccLen);

        ConnMgrModemCarrierMcc_t pmccValue = {
                pMcc,
                &pMccLen,
        };
        
        char pMnc[1024];
        uint32_t pMncLen = 1024;
        memset(pMnc, 0, pMncLen);

        ConnMgrModemCarrierMcc_t pmncValue = {
                pMnc,
                &pMncLen,
        };
        
        char pOperatorName[1024];
        uint32_t pOperatorNameLen = 1024;
        memset(pOperatorName, 0, pOperatorNameLen);

        ConnMgrModemCarrierName_t pnameValue = {
                pOperatorName,
                &pOperatorNameLen,
        };

        ConnMgrModemCarrierInfo_t pValue = {
                &pmccValue,
                &pmncValue,
                &pnameValue
        };
        
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemCheckCarrier(index, &pValue);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("Mcc:%s\n", pValue.pMcc->pMcc);
            printf("Mcc:%s\n", pValue.pMcc->pMcc);
            LOG_FILE("%d\n", *(pValue.pMcc->pMccLen));
            LOG_FILE("Mnc:%s\n", pValue.pMnc->pMnc);
            printf("Mnc:%s\n", pValue.pMnc->pMnc);
            LOG_FILE("%d\n", *(pValue.pMnc->pMncLen));
            LOG_FILE("OperatorName:%s\n", pValue.pName->pOperatorName);
            printf("OperatorName:%s\n", pValue.pName->pOperatorName);
            LOG_FILE("%d\n", *(pValue.pName->pOperatorNameLen));
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "switchcarrier") == 0) { 
        char pMcc[1024];
        uint32_t pMccLen = 1024;
        memset(pMcc, 0, pMccLen);
        strncpy (pMcc, argv[4], strlen(argv[4]));
        pMccLen = strlen(pMcc);
        ConnMgrModemCarrierMcc_t pSMcc = {
                pMcc,
                &pMccLen
        };
        
        char pMnc[1024];
        uint32_t pMncLen = 1024;
        memset(pMnc, 0, pMncLen);
        strncpy (pMnc, argv[5], strlen(argv[5]));
        pMncLen = strlen(pMnc);
        ConnMgrModemCarrierMcc_t pSMnc = {
                pMnc,
                &pMncLen
        };
        
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d, mcc:%s, mnc:%s\n", argv[2], index, argv[4], argv[5]);
        StatusCode = AsusConnMgrModemSwitchCarrier(index, &pSMcc, &pSMnc);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "geticcid") == 0) {
        char pRBuffer[1024];
        uint32_t pRBufLen = 1024;
        memset(pRBuffer, 0, pRBufLen);
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemGetICCID(index, pRBuffer, &pRBufLen);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("%s\n", pRBuffer);
            printf("%s\n", pRBuffer);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getimsi") == 0) {
        char pRBuffer[1024];
        uint32_t pRBufLen = 1024;
        memset(pRBuffer, 0, pRBufLen);
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemGetIMSI(index, pRBuffer, &pRBufLen);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("%s\n", pRBuffer);
            printf("%s\n", pRBuffer);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getsignalstrength") == 0) {
        uint32_t pStrength = 0;
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemGetSignalStrength(index, &pStrength);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("%d\n", pStrength);
            printf("%d\n", pStrength);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getkeepalivestatus") == 0) {
        uint32_t pValue = 0;
        LOG_FILE("eapiModemMgmt() %s\n", argv[2]);
        StatusCode = AsusConnMgrModemGetKeepAliveStatus(&pValue);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("%d\n", pValue);
            printf("%d\n", pValue);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getfailoverstatus") == 0) {
        uint32_t pValue = 0;
        LOG_FILE("eapiModemMgmt() %s\n", argv[2]);
        StatusCode = AsusConnMgrGetFailoverStatus(&pValue);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("%d\n", pValue);
            printf("%d\n", pValue);
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getprofile") == 0) {
        char pApn[1024];
        uint32_t pApnLen = 1024;
        memset(pApn, 0, pApnLen);
        
        char pUser[1024];
        uint32_t pUserLen = 1024;
        memset(pUser, 0, pUserLen);
        
        char pPassword[1024];
        uint32_t pPasswordLen = 1024;
        memset(pPassword, 0, pPasswordLen);
        
        char pIpv4Method[1024];
        uint32_t pIpv4MethodLen = 1024;
        memset(pIpv4Method, 0, pIpv4MethodLen);
        
        char pIpv6Method[1024];
        uint32_t pIpv6MethodLen = 1024;
        memset(pIpv6Method, 0, pIpv6MethodLen);

        ConnMgrModemProfile_t pValue = {
            
                pApn,
                &pApnLen,
                pUser,
                &pUserLen,
                pPassword,
                &pPasswordLen,
                pIpv4Method,
                &pIpv4MethodLen,
                pIpv6Method,
                &pIpv6MethodLen
            
        };
        
        LOG_FILE("eapiModemMgmt() %s\n", argv[2]);
        StatusCode = AsusConnMgrModemGetProfile(&pValue);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("Apn:%s\n", pValue.pApn);
            printf("Apn:%s\n", pValue.pApn);
            LOG_FILE("%d\n", *(pValue.pApnLen));
            LOG_FILE("UserName:%s\n", pValue.pUser);
            printf("UserName:%s\n", pValue.pUser);
            LOG_FILE("%d\n", *(pValue.pUserLen));
            LOG_FILE("Password:%s\n", pValue.pPassword);
            printf("Password:%s\n", pValue.pPassword);
            LOG_FILE("%d\n", *(pValue.pPasswordLen));
            LOG_FILE("IPv4Method:%s\n", pValue.pIpv4Method);
            printf("IPv4Method:%s\n", pValue.pIpv4Method);
            LOG_FILE("%d\n", *(pValue.pIpv4MethodLen));
            LOG_FILE("IPv6Method:%s\n", pValue.pIpv6Method);
            printf("IPv6Method:%s\n", pValue.pIpv6Method);
            LOG_FILE("%d\n", *(pValue.pIpv6MethodLen));
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getfailovergroup") == 0) {
        uint32_t cntRow = 30;
        uint32_t cntColumn = 256;
        char **pInterface;
        pInterface = (char**)malloc(sizeof(char*)*cntRow);
        for(int i=0; i<cntRow; i++)
        {
            pInterface[i] = (char*)malloc(sizeof(char)*cntColumn);
            
        }
        uint32_t inCntRow = cntRow;
        LOG_FILE("eapiModemMgmt() %s, cntRow:%d, cntColumn:%d\n", argv[2], cntRow, cntColumn);
        StatusCode = AsusConnMgrGetFailoverGroup(pInterface, &inCntRow, cntColumn);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            if(inCntRow <= 0) {
                LOG_FILE("NumberOfInterfaces:%d\n", inCntRow);
                printf("NumberOfInterfaces:%d\n", inCntRow);
            } else {
                LOG_FILE("inCntRow:%d\n", inCntRow);
                for(int i=0; i<inCntRow; i++) {
                    LOG_FILE("pInterface[%d]:%s\n", i, pInterface[i]);
                    printf("pInterface[%d]:%s\n", i, pInterface[i]);
                    LOG_FILE("strlen:%d\n", strlen(pInterface[i]));
                }
            }
        } else {
            parseErrorCode(StatusCode);
        }
        LOG_FILE("cntRow:%d\n", cntRow);
        for(int i=0; i<cntRow; i++) {
            free(pInterface[i]);
            pInterface[i] = NULL;
        }
        free(pInterface);
        pInterface = NULL;
    } else if(strcmp(argv[2], "cleanfailovergroup") == 0) {
        int cntRow = 0;
        char **pInterface = NULL;
        LOG_FILE("eapiModemMgmt() %s\n", argv[2]);
        StatusCode = AsusConnMgrSetFailoverGroup(pInterface, cntRow);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "setfailovergroup") == 0) {
        int cntRow = argc - 3;
        uint32_t cntColumn = 256;
        char **pInterface;
        pInterface = (char**)malloc(sizeof(char*)*cntRow);
        LOG_FILE("setfailovergroup() cntRow:%d,cntColumn:%d\n", cntRow, cntColumn);
        for(int i=0; i<cntRow; i++) {
            pInterface[i] = (char*)malloc(sizeof(char)*cntColumn);
            memset(pInterface[i], 0, sizeof(char)*cntColumn);
            strcpy(pInterface[i], argv[i+3]);
            LOG_FILE("setfailovergroup() pInterface[%d]:%s\n", i, pInterface[i]);
        }
        LOG_FILE("eapiModemMgmt() %s, cntRow:%d\n", argv[2], cntRow);
        StatusCode = AsusConnMgrSetFailoverGroup(pInterface, cntRow);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("PASS\n");
            printf("PASS\n");
        } else {
            parseErrorCode(StatusCode);
        }
    } else if(strcmp(argv[2], "getcelllocationinfo") == 0) {
        char pApn[1024];
        uint32_t pApnLen = 1024;
        memset(pApn, 0, pApnLen);
        
        char pUser[1024];
        uint32_t pUserLen = 1024;
        memset(pUser, 0, pUserLen);
        
        char pPassword[1024];
        uint32_t pPasswordLen = 1024;
        memset(pPassword, 0, pPasswordLen);
        
        char pIpv4Method[1024];
        uint32_t pIpv4MethodLen = 1024;
        memset(pIpv4Method, 0, pIpv4MethodLen);
        
        char pIpv6Method[1024];
        uint32_t pIpv6MethodLen = 1024;
        memset(pIpv6Method, 0, pIpv6MethodLen);

        ConnMgrModemCellLocationInfo_t pValue = {
                pApn,
                &pApnLen,
                pUser,
                &pUserLen,
                pPassword,
                &pPasswordLen,
                pIpv4Method,
                &pIpv4MethodLen,
                pIpv6Method,
                &pIpv6MethodLen
            
        };
        
        uint32_t index = (int)strtol(argv[3], NULL, 0);
        LOG_FILE("eapiModemMgmt() %s, index:%d\n", argv[2], index);
        StatusCode = AsusConnMgrModemGetCellLocationInfo(index, &pValue);
        if(StatusCode == ASUS_API_STATUS_SUCCESS) {
            LOG_FILE("OperatorCode:%s\n", pValue.pOperatorCode);
            printf("OperatorCode:%s\n", pValue.pOperatorCode);
            LOG_FILE("%d\n", *(pValue.pOperatorCodeLen));
            LOG_FILE("OperatorName:%s\n", pValue.pOperatorName);
            printf("OperatorName:%s\n", pValue.pOperatorName);
            LOG_FILE("%d\n", *(pValue.pOperatorNameLen));
            LOG_FILE("LocationAreaCode:%s\n", pValue.pLocationAreaCode);
            printf("LocationAreaCode:%s\n", pValue.pLocationAreaCode);
            LOG_FILE("%d\n", *(pValue.pLocationAreaCodeLen));
            LOG_FILE("TrackingAreaCode:%s\n", pValue.pTrackingAreaCode);
            printf("TrackingAreaCode:%s\n", pValue.pTrackingAreaCode);
            LOG_FILE("%d\n", *(pValue.pTrackingAreaCodeLen));
            LOG_FILE("CellId:%s\n", pValue.pCellId);
            printf("CellId:%s\n", pValue.pCellId);
            LOG_FILE("%d\n", *(pValue.pCellIdLen));
        } else {
            parseErrorCode(StatusCode);
        }
    }
    
    LOG_FILE("eapiModemMgmt() StatusCode:%d\n", StatusCode);
    EApiStatus_t retStatusCode = EApiLibUnInitialize();
    if(retStatusCode != ASUS_API_STATUS_SUCCESS) {
        LOG_FILE("eapiModemMgmt() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        printf("eapiModemMgmt() FAIL on eApiLibUnInitialize:%d\n", retStatusCode);
        if(StatusCode != ASUS_API_STATUS_SUCCESS) {
            return StatusCode;
        } else {
            return retStatusCode;
        }
    } else {
        LOG_FILE("eapiModemMgmt() eApiLibUnInitialize return success\n");
    }
    return StatusCode;
}

int main(int argc, char *argv[])
{
    int c;
    char *hValue = NULL;
    while ((c = getopt (argc, argv, "h:")) != -1) {
        switch(c) {
            case 'h':
                hValue = optarg;
                printCmdHelp(hValue);
                return 0;
            break;
            default:
            break;
        }
    }
    if(argc < 2) {
        LOGFILE_STDOUT ("ERROR on argc:%d\n", argc);
    }
    if(strcmp(argv[1], "-h") == 0) {
        printHelp();
        return 0;
    }

    if(strcmp(argv[1], "getstring") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with getstring:%d\n", argc);
            return -1;
        }
        enableLog();
        EApiId_t id = atoi(argv[2]);
        int iRet = eapiGetString(id);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "getvalue") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with getvalue:%d\n", argc);
            return -1;
        }
        enableLog();
        EApiId_t id = atoi(argv[2]);
        int iRet = eapiGetValue(id);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "setvalue") == 0) {
        if(argc < 4) {
            LOGFILE_STDOUT ("ERROR on argc with setvalue:%d\n", argc);
            return -1;
        }
        enableLog();
        EApiId_t id = atoi(argv[2]);
        uint32_t value = atoi(argv[3]);
        int iRet = eapiSetValue(id, value);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "led") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with led:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "turnon") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with turnon:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getnumberofleds") == 0) {
        } else if(strcmp(argv[2], "setoccupied") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with setoccupied:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "turnoff") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with turnoff:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getcap") == 0) {
        } else {
            LOGFILE_STDOUT ("ERROR on argv with led:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = eapiLed(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "systemboot") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with systemboot:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "get") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with get:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "set") == 0) {
            if(argc < 10) {
                LOGFILE_STDOUT ("ERROR on argc with set:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with systemboot:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = eapiSystemBoot(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "watchdog") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with watchdog:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "getcap") == 0) {
        } else if(strcmp(argv[2], "start") == 0) {
            if(argc < 6) {
                LOGFILE_STDOUT ("ERROR on argc with watchdog start:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with watchdog:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = eapiWatchdog(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "vcprtc") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with vcprtc:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "set") == 0) {
            if(argc < 10) {
                LOGFILE_STDOUT ("ERROR on argc with set:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "get") == 0) {
            if(argc < 3) {
                LOGFILE_STDOUT ("ERROR on argc with get:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with vcprtc:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = EApiVcpRtc(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "vcppowermgmt") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with vcppowermgmt:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "setlbpthreshold") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setlbpthreshold:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getlbpthreshold") == 0) {
            if(argc < 3) {
                LOGFILE_STDOUT ("ERROR on argc with getlbpthreshold:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "seteventdelay") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with seteventdelay:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "geteventdelay") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with geteventdelay:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with vcppowermgmt:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = EApiVcpPowerMgmt(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "vcpwakeup") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with vcpwakeup:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "getenabledwakeuptype") == 0) {
            if(argc < 3) {
                LOGFILE_STDOUT ("ERROR on argc with getenabledwakeuptype:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getlastwakeupsource") == 0) {
            if(argc < 3) {
                LOGFILE_STDOUT ("ERROR on argc with getlastwakeupsource:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "enable") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with enable:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "disable") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with disable:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with vcpwakeup:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = EApiVcpWakeup(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "vcpcomport") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with vcpcomport:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "enable") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with enable:%d\n", argc);
                return -1;
            }
            uint32_t id = atoi(argv[3]);
            if(id >= 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with enable:%d\n", id);
                return -1;
            }
        } else if(strcmp(argv[2], "disable") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with disable:%d\n", argc);
                return -1;
            }
            uint32_t id = atoi(argv[3]);
            if(id >= 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with disable:%d\n", id);
                return -1;
            }
        } else if(strcmp(argv[2], "init") == 0) {
            if(argc < 10) {
                LOGFILE_STDOUT ("ERROR on argc with init:%d\n", argc);
                return -1;
            }
            uint32_t id = atoi(argv[3]);
            if(id >= 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with init:%d\n", id);
                return -1;
            }
        } else if(strcmp(argv[2], "read") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with read:%d\n", argc);
                return -1;
            }
            uint32_t id = atoi(argv[3]);
            if(id >= 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with read:%d\n", id);
                return -1;
            }
        } else if(strcmp(argv[2], "write") == 0) {
            if(argc < 6) {
                LOGFILE_STDOUT ("ERROR on argc with write:%d\n", argc);
                return -1;
            }
            uint32_t id = atoi(argv[3]);
            if(id >= 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with write:%d\n", id);
                return -1;
            }
            uint32_t ByteCnt = (uint32_t)strtol(argv[5], NULL, 0);
            if(ByteCnt > 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with write ByteCnt:%d\n", ByteCnt);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with vcpcomport:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = EApiVcpComPort(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "vcpsensor") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with vcpsensor:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "read") == 0) {
        } else if(strcmp(argv[2], "getoffset") == 0) {
        } else if(strcmp(argv[2], "setoffset") == 0) {
            if(argc < 6) {
                LOGFILE_STDOUT ("ERROR on argc with setoffset:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "enable") == 0) {
        } else if(strcmp(argv[2], "disable") == 0) {
        } else if(strcmp(argv[2], "getresolution") == 0) {
        } else if(strcmp(argv[2], "getalarmthreshold") == 0) {
        } else if(strcmp(argv[2], "setalarmthreshold") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with setalarmthreshold:%d\n", argc);
                return -1;
            }
            char *argFirst;
            argFirst = argv[3];
            if(strcmp(argFirst, "2g") == 0) {
            } else if(strcmp(argFirst, "4g") == 0) {
            } else if(strcmp(argFirst, "8g") == 0) {
            } else if(strcmp(argFirst, "16g") == 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with setalarmthreshold:%s\n", argFirst);
                return -1;
            }
        } else if(strcmp(argv[2], "enablealarm") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with enablealarm:%d\n", argc);
                return -1;
            }
            char *argFirst;
            argFirst = argv[3];
            if(strcmp(argFirst, "x") == 0) {
            } else if(strcmp(argFirst, "y") == 0) {
            } else if(strcmp(argFirst, "z") == 0) {
            } else if(strcmp(argFirst, "xy") == 0) {
            } else if(strcmp(argFirst, "xz") == 0) {
            } else if(strcmp(argFirst, "yz") == 0) {
            } else if(strcmp(argFirst, "xyz") == 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with enablealarm:%s\n", argFirst);
                return -1;
            }
        } else if(strcmp(argv[2], "disablealarm") == 0) {
        } else if(strcmp(argv[2], "getalarm") == 0) {
        } else if(strcmp(argv[2], "available") == 0) {
        } else if(strcmp(argv[2], "getstatus") == 0) {
        } else if(strcmp(argv[2], "setresolution") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setresolution:%d\n", argc);
                return -1;
            }
            char *argFirst;
            argFirst = argv[3];
            if(strcmp(argFirst, "2g") == 0) {
            } else if(strcmp(argFirst, "4g") == 0) {
            } else if(strcmp(argFirst, "8g") == 0) {
            } else if(strcmp(argFirst, "16g") == 0) {
            } else {
                LOGFILE_STDOUT ("ERROR on argc with setresolution:%s\n", argFirst);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with vcpsensor:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = EApiVcpGSensor(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "i2c") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with i2c:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "getbuscap") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getbuscap:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "probedevice") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with probedevice:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "readtransfer") == 0) {
            if(argc < 7) {
                LOGFILE_STDOUT ("ERROR on argc with readtransfer:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "writetransfer") == 0) {
            if(argc < 8) {
                LOGFILE_STDOUT ("ERROR on argc with writetransfer:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "writereadraw") == 0) {
            if(argc < 8) {
                LOGFILE_STDOUT ("ERROR on argc with writereadraw:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "readraw") == 0) {
            if(argc < 6) {
                LOGFILE_STDOUT ("ERROR on argc with readraw:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "writeraw") == 0) {
            if(argc < 7) {
                LOGFILE_STDOUT ("ERROR on argc with writeraw:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with i2c:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = eapiI2C(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "cm") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with cm:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "startnetwork") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with startnetwork:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "stopnetwork") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with stopnetwork:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getattachedstatus") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getattachedstatus:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getstatus") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getstatus:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "poweron") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with poweron:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "poweroff") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with poweroff:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "restart") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with restart:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "checkcarrier") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with checkcarrier:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "geticcid") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with geticcid:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getimsi") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getimsi:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getsignalstrength") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getsignalstrength:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getcelllocationinfo") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getcelllocationinfo:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getadvancedsignalinfo") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getadvancedsignalinfo:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getmodeminfo") == 0) {
        } else if(strcmp(argv[2], "getnumberofmodems") == 0) {
        } else if(strcmp(argv[2], "cleanfailovergroup") == 0) {
        } else if(strcmp(argv[2], "setfailovergroup") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setfailovergroup:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getfailovergroup") == 0) {
        } else if(strcmp(argv[2], "resetprofile") == 0) {
        } else if(strcmp(argv[2], "setfailover") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setfailover:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setkeepalive") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setkeepalive:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getprofile") == 0) {
        } else if(strcmp(argv[2], "getfailoverstatus") == 0) {
        } else if(strcmp(argv[2], "getkeepalivestatus") == 0) {
        } else if(strcmp(argv[2], "switchsim") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with switchsim:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "unlockpin") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with unlockpin:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setflightmode") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with setflightmode:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setiptype") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setiptype:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setapn") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setapn:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setuser") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setuser:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setpassword") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with setpassword:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "switchcarrier") == 0) {
            if(argc < 6) {
                LOGFILE_STDOUT ("ERROR on argc with switchcarrier:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with gpio:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = eapiModemMgmt(argc, argv);
        disableLog();
        return iRet;
    } else if(strcmp(argv[1], "gpio") == 0) {
        if(argc < 3) {
            LOGFILE_STDOUT ("ERROR on argc with gpio:%d\n", argc);
            return -1;
        }
        if(strcmp(argv[2], "getcap") == 0) {
            if(argc < 4) {
                LOGFILE_STDOUT ("ERROR on argc with getcap:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getdirection") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with getdirection:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "getlevel") == 0) {
            if(argc < 5) {
                LOGFILE_STDOUT ("ERROR on argc with getlevel:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setdirection") == 0) {
            if(argc < 6) {
                LOGFILE_STDOUT ("ERROR on argc with setdirection:%d\n", argc);
                return -1;
            }
        } else if(strcmp(argv[2], "setlevel") == 0) {
            if(argc < 6) {
                LOGFILE_STDOUT ("ERROR on argc with setlevel:%d\n", argc);
                return -1;
            }
        } else {
            LOGFILE_STDOUT ("ERROR on argv with gpio:%s\n", argv[2]);
            return -1;
        }
        enableLog();
        int iRet = eapiGpio(argc, argv);
        disableLog();
        return iRet;
    } else {
        LOGFILE_STDOUT("Unknown arg:%s\n", argv[1]);
        return -1;
    }
    return 0;
}

