#ifndef _EAPI_H_        
#define _EAPI_H_

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************************/

#ifndef __IN
#  define __IN 
/* __IN
 * Arg Type        | Characteristics
 * ----------------+-----------------------------------------------------
 * Immediate value | Input value that must be specified and is essential 
 *                 | to function operation.
 *                 | 
 * Pointer         | Valid pointer to initialized buffer/variable.
 *                 | 
 */
#endif
#ifndef __INOPT
#  define __INOPT
/* __INOPT
 * Arg Type        | Characteristics
 * ----------------+-----------------------------------------------------
 * Pointer         | Valid pointer to initialized buffer/variable, or
 *                 | NULL Pointer.
 *                 | Note: refer to function specification for specifics.
 */
#endif
#ifndef __OUT
#  define __OUT
/* __OUT
 * Arg Type        | Characteristics
 * ----------------+-----------------------------------------------------
 * Pointer         | Valid pointer to destination buffer/variable
 */
#endif
#ifndef __OUTOPT
#  define __OUTOPT

/* __OUTOPT
 * Arg Type        | Characteristics
 * ----------------+-----------------------------------------------------
 * Pointer         | Valid pointer to destination buffer/variable, or
 *                 | NULL Pointer. 
 *                 | Note: refer to function specification for specifics.
 */
#endif
#ifndef __INOUT
#  define __INOUT
/* __INOUT
 * Arg Type        | Characteristics
 * ----------------+-----------------------------------------------------
 * Pointer         | Valid pointer to initialized buffer/variable
 *                 | Contents of buffer/variable updated before return.
 */
#endif
#ifndef EAPI_CALLTYPE
#if defined(_WIN64) || defined(_WIN32)
#  define EAPI_CALLTYPE __declspec(dllexport)
#  define ASUS_CALLTYPE __declspec(dllexport)
#else
#  define EAPI_CALLTYPE __attribute__ ((visibility("default")))
#  define ASUS_CALLTYPE __attribute__ ((visibility("default")))
#endif
#endif
#ifndef EAPI_UINT32_C
#  define EAPI_UINT8_C(x)  ((uint8_t)(x))
#  define EAPI_UINT16_C(x) ((uint16_t)(x))
#  define EAPI_UINT32_C(x) ((uint32_t)(x))
#endif
/************************************************************************
 * All version numbers 
 * +=========+===============================+
 * | Bits    | Descriptions                  |
 * +=========+===============================+
 * | 24 - 31 | Version Number                |
 * +---------+-------------------------------+
 * | 16 - 23 | Revision Number               |
 * +---------+-------------------------------+
 * | 0  - 15 | Build Number                  |
 * +=========+===============================+
 */
#define EAPI_VER_MASK_VER     EAPI_UINT32_C(0xFF000000)
#define EAPI_VER_MASK_REV     EAPI_UINT32_C(0x00FF0000)
#define EAPI_VER_MASK_BUILD   EAPI_UINT32_C(0x0000FFFF)
#define EAPI_VER_GET_VER(x)   EAPI_UINT8_C (((x)>>24)&UINT8_MAX )
#define EAPI_VER_GET_REV(x)   EAPI_UINT8_C (((x)>>16)&UINT8_MAX )
#define EAPI_VER_GET_BUILD(x) EAPI_UINT16_C(((x)>> 0)&UINT16_MAX)
#define EAPI_VER_CREATE(Version,Revision,Build) (\
                      EAPI_UINT32_C(\
                        (((Version )&UINT8_MAX )<<24)|\
                        (((Revision)&UINT8_MAX )<<16)|\
                        (((Build   )&UINT16_MAX)<< 0) \
                      )\
                    )

/* Embedded API Standard Revision */
#define EAPI_VER      1
#define EAPI_REVISION 0
#define EAPI_VERSION EAPI_VER_CREATE(EAPI_VER, EAPI_REVISION, 0)


/************************************************************************
 */


/* Embedded Application System Interface */

/*
 *  EApi Types
 */
typedef uint32_t EApiStatus_t;
typedef uint32_t EApiId_t;
typedef uint32_t AsusStatus_t;

/*
 *
 *
 *  S T A T U S   C O D E S
 *
 *
 */
/* Description
 *   The EAPI library is not yet or unsuccessfully initialized. 
 *   EApiLibInitialize needs to be called prior to the first access of any 
 *   other EAPI function.
 * Actions
 *   Call EApiLibInitialize..
 */
#define EAPI_STATUS_NOT_INITIALIZED   EAPI_UINT32_C(0xFFFFFFFF)

/* Description
 *   Library is initialized.
 * Actions
 *   none.
 */
#define EAPI_STATUS_INITIALIZED       EAPI_UINT32_C(0xFFFFFFFE)

/* Description
 *   Memory Allocation Error.
 * Actions
 *   Free memory and try again..
 */
#define EAPI_STATUS_ALLOC_ERROR       EAPI_UINT32_C(0xFFFFFFFD)

/* Description 
 *   Time out in driver. This is Normally caused by hardware/software 
 *   semaphore timeout. 
 * Actions
 *   Retry.
 */
#define EAPI_STATUS_DRIVER_TIMEOUT    EAPI_UINT32_C(0xFFFFFFFC)

/* Description 
 * Hardware maybe not ready.
 * Actions
 * Try to verify BIOS setting.
 */
#define EAPI_STATUS_DEVICE_NOT_READY    EAPI_UINT32_C(0xFFFFFFFB)

/* Description 
 *  One or more of the EAPI function call parameters are out of the 
 *  defined range.
 *
 *  Possible Reasons include be
 *  NULL Pointer
 *  Invalid Offset
 *  Invalid Length
 *  Undefined Value
 *  
 *   Storage Write
 *    Incorrectly Aligned Offset
 *    Invalid Write Length
 * Actions
 *   Verify Function Parameters.
 */
#define EAPI_STATUS_INVALID_PARAMETER EAPI_UINT32_C(0xFFFFFEFF)

/* Description
 *   The Block Alignment is incorrect.
 * Actions
 *   Use pInputs and pOutputs to correctly select input and outputs. 
 */
#define EAPI_STATUS_INVALID_BLOCK_ALIGNMENT EAPI_UINT32_C(0xFFFFFEFE)

/* Description
 *   This means that the Block length is too long.
 * Actions
 *   Use Alignment Capabilities information to correctly align write access.
 */
#define EAPI_STATUS_INVALID_BLOCK_LENGTH    EAPI_UINT32_C(0xFFFFFEFD)

/* Description
 *   The current Direction Argument attempts to set GPIOs to a unsupported 
 *   directions. I.E. Setting GPI to Output.
 * Actions
 *   Use pInputs and pOutputs to correctly select input and outputs. 
 */
#define EAPI_STATUS_INVALID_DIRECTION       EAPI_UINT32_C(0xFFFFFEFC)

/* Description
 *   The Bitmask Selects bits/GPIOs which are not supported for the current ID.
 * Actions
 *   Use pInputs and pOutputs to probe supported bits..
 */
#define EAPI_STATUS_INVALID_BITMASK         EAPI_UINT32_C(0xFFFFFEFB)

/* Description
 *   Watchdog timer already started.
 * Actions
 *   Call EApiWDogStop, before retrying.
 */
#define EAPI_STATUS_RUNNING                 EAPI_UINT32_C(0xFFFFFEFA)

/* Description
 *   This function or ID is not supported at the actual hardware environment.
 * Actions
 *   none.
 */
#define EAPI_STATUS_UNSUPPORTED       EAPI_UINT32_C(0xFFFFFCFF)

/* Description
 *   I2C Device Error
 *   No Acknowledge For Device Address, 7Bit Address Only
 *   10Bit Address may cause Write error if 2 10Bit addressed devices 
 *   present on the bus.
 * Actions
 *   none.
 */
#define EAPI_STATUS_NOT_FOUND         EAPI_UINT32_C(0xFFFFFBFF)

/* Description
 *   I2C Time-out
 *   Device Clock stretching time-out, Clock pulled low by device 
 *   for too long
 * Actions
 *   none.
 */
#define EAPI_STATUS_TIMEOUT           EAPI_UINT32_C(0xFFFFFBFE)

/* Description
 *   EApi  I2C functions specific. The addressed I2C bus is busy or there 
 *   is a bus collision.
 *   The I2C bus is in use. Either CLK or DAT are low.
 *   Arbitration loss or bus Collision, data remains low when writing a 1
 * Actions
 *   Retry.
 */
#define EAPI_STATUS_BUSY_COLLISION    EAPI_UINT32_C(0xFFFFFBFD)

/* Description
 *   I2C Read Error
 *    Not Possible to detect.
 *   Storage Read Error
 *    ....
 * Actions
 *   Retry.
 */
#define EAPI_STATUS_READ_ERROR        EAPI_UINT32_C(0xFFFFFAFf)

/* Description
 *   I2C Write Error
 *     No Acknowledge received after writing any Byte after the First Address 
 *     Byte.
 *     Can be caused by 
 *     unsupported Device Command/Index
 *     Ext Command/Index used on Standard Command/Index Device
 *     10Bit Address Device Not Present
 *   Storage Write Error
 *    ...
 * Actions
 *   Retry.
 */
#define EAPI_STATUS_WRITE_ERROR       EAPI_UINT32_C(0xFFFFFAFE)


/* Description
 *   The amount of available data exceeds the buffer size.
 *   Storage buffer overflow was prevented. Read count was larger then 
 *   the defined buffer length.
 *   Read Count > Buffer Length
 * Actions
 *   Either increase the buffer size or reduce the block length.
 */
#define EAPI_STATUS_MORE_DATA         EAPI_UINT32_C(0xFFFFF9FF)

/* Description
 *   Generic error message. No further error details are available.
 * Actions
 *   none.
 */
#define EAPI_STATUS_ERROR             EAPI_UINT32_C(0xFFFFF0FF)

/* Description
 *   The operation was successful.
 * Actions
 *   none.
 */
#define EAPI_STATUS_SUCCESS           EAPI_UINT32_C(0)
#define EAPI_TEST_SUCCESS(x)        (!(x))

#define ASUS_API_STATUS_NOT_INITIALIZED   EAPI_UINT32_C(0xFFFFFFFF)
#define ASUS_API_STATUS_INITIALIZED       EAPI_UINT32_C(0xFFFFFFFE)
#define ASUS_API_STATUS_ALLOC_ERROR       EAPI_UINT32_C(0xFFFFFFFD)
#define ASUS_API_STATUS_DRIVER_TIMEOUT    EAPI_UINT32_C(0xFFFFFFFC)
#define ASUS_API_STATUS_DEVICE_NOT_READY    EAPI_UINT32_C(0xFFFFFFFB)
#define ASUS_API_STATUS_INVALID_PARAMETER EAPI_UINT32_C(0xFFFFFEFF)
#define ASUS_API_STATUS_INVALID_BLOCK_ALIGNMENT EAPI_UINT32_C(0xFFFFFEFE)
#define ASUS_API_STATUS_INVALID_BLOCK_LENGTH    EAPI_UINT32_C(0xFFFFFEFD)
#define ASUS_API_STATUS_INVALID_DIRECTION       EAPI_UINT32_C(0xFFFFFEFC)
#define ASUS_API_STATUS_INVALID_BITMASK         EAPI_UINT32_C(0xFFFFFEFB)
#define ASUS_API_STATUS_RUNNING                 EAPI_UINT32_C(0xFFFFFEFA)
#define ASUS_API_STATUS_UNSUPPORTED       EAPI_UINT32_C(0xFFFFFCFF)
#define ASUS_API_STATUS_NOT_FOUND         EAPI_UINT32_C(0xFFFFFBFF)
#define ASUS_API_STATUS_TIMEOUT           EAPI_UINT32_C(0xFFFFFBFE)
#define ASUS_API_STATUS_BUSY_COLLISION    EAPI_UINT32_C(0xFFFFFBFD)
#define ASUS_API_STATUS_READ_ERROR        EAPI_UINT32_C(0xFFFFFAFf)
#define ASUS_API_STATUS_WRITE_ERROR       EAPI_UINT32_C(0xFFFFFAFE)
#define ASUS_API_STATUS_MORE_DATA         EAPI_UINT32_C(0xFFFFF9FF)
#define ASUS_API_STATUS_ERROR             EAPI_UINT32_C(0xFFFFF0FF)

#define ASUS_API_STATUS_INVALID_PERMISSION_ACCESS EAPI_UINT32_C(0xFFAFFFFF)

#define ASUS_API_STATUS_SUCCESS           EAPI_UINT32_C(0)
#define ASUS_TEST_SUCCESS(x)        (!(x))


/* Library */
/* 
 * EApiLibInitialize
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Already initialized            | EAPI_STATUS_INITIALIZED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiLibInitialize(void)   ; /* Should be called before 
                             * calling any other API 
                             * function is called
                             */
/* 
 * EApiLibUnInitialize
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE 
EApiLibUnInitialize(void) ; /* Should be called before 
                             * program exit 
                             */ 


/* 
 *
 * Plug and Play Identifiers
 *
 *
 *
 */
/* 
 * The following MACROS are for Creating OEM IDs
 * OEM ID macros should be named
 * EAPI_[PNPID]_ID_[TYPE]_[NAME] 
 * E.G.
 * EAPI_PMG_ID_BOARD_CUSTOMERID
 */
#define EAPI_BYTE_SWAP_W(a) EAPI_UINT16_C(\
      (((a)<<8)&0xFF00)|\
      (((a)>>8)&0x00FF) \
      )

#define EAPI_CREATE_PNPID(a, b, c) \
  EAPI_BYTE_SWAP_W(((( a -'A'+1)&0x1F)<<10)|\
              ((( b -'A'+1)&0x1F)<< 5)|\
              ((( c -'A'+1)&0x1F)<< 0) \
             )
#define EAPI_CREATE_CUST_ID(a, b, c, Id)\
      EAPI_UINT32_C((0xF<<28)|(EAPI_CREATE_PNPID(a, b, c)<<12)|(Id&0xFFF))

#define EAPI_PNPID_PICMG EAPI_CREATE_PNPID('P', 'M', 'G')  /* PICMG Should
                                                            * register this. 
                                                            */

/*
 *
 *      B O A R D   I N F O M A T I O N   S T R I N G S 
 *
 */
/* IDS */
#define EAPI_ID_BOARD_MANUFACTURER_STR     EAPI_UINT32_C(0) /* Board Manufacturer
                                                             * Name String 
                                                             */
#define EAPI_ID_BOARD_NAME_STR             EAPI_UINT32_C(1) /* Board Name String */
#define EAPI_ID_BOARD_REVISION_STR         EAPI_UINT32_C(2) /* Board Name String */
#define EAPI_ID_BOARD_SERIAL_STR           EAPI_UINT32_C(3) /* Board Serial 
                                 * Number String 
                                 */
#define EAPI_ID_BOARD_BIOS_REVISION_STR    EAPI_UINT32_C(4) /* Board Bios Revision
                                 * String 
                                 */
#define EAPI_ID_BOARD_HW_REVISION_STR      EAPI_UINT32_C(5) /* Board Hardware 
                                 * Revision String 
                                 */
#define EAPI_ID_BOARD_PLATFORM_TYPE_STR    EAPI_UINT32_C(6) /* Platform ID 
                                 * (ETX, COM Express, 
                                 * etc...) 
                                 */


/* 
 * EApiBoardGetStringA
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * pBufLen==NULL                          | EAPI_STATUS_INVALID_PARAMETER
 * pBufLen!=NULL&&*pBufLen&&pBuffer==NULL | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * strlen(Id)+1>*pBufLen                  | EAPI_STATUS_MORE_DATA
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE
EApiBoardGetStringA(
    __IN      EApiId_t  Id      , /* Name Id */
    __OUT     char     *pBuffer , /* Destination pBuffer */
    __INOUT   uint32_t *pBufLen   /* pBuffer Length */
    );


/*
 *
 *      B O A R D   I N F O M A T I O N   V A L U E S
 *
 */
/* IDS */
#define EAPI_ID_GET_EAPI_SPEC_VERSION         EAPI_UINT32_C(0) /* EAPI Specification 
                                                                * Revision I.E. The 
                                                                * EAPI Spec Version 
                                                                * Bits 31-24, Revision 
                                                                * 23-16, 15-0 always 0
                                                                * Used to implement 
                                                                * this interface
                                                                */

#define EAPI_ID_BOARD_BOOT_COUNTER_VAL        EAPI_UINT32_C(1) /* Units = Boots */
#define EAPI_ID_BOARD_RUNNING_TIME_METER_VAL  EAPI_UINT32_C(2) /* Units = Minutes */
#define EAPI_ID_BOARD_PNPID_VAL               EAPI_UINT32_C(3) /* Encoded PNP ID 
                                                                * Format 
                                                                * (Compressed ASCII)
                                                                */
#define EAPI_ID_BOARD_PLATFORM_REV_VAL        EAPI_UINT32_C(4) /* Platform Revision 
                                                                * I.E. The PICMG Spec 
                                                                * Version Bits 31-24,
                                                                * Revision 23-16, 
                                                                * 15-0 always 0
                                                                */

#define EAPI_ID_BOARD_DRIVER_VERSION_VAL  EAPI_UINT32_C(0x10000) /* Vendor Specific 
                                                                  * (Optional) 
                                                                  */
#define EAPI_ID_BOARD_LIB_VERSION_VAL     EAPI_UINT32_C(0x10001) /* Vendor Specific
                                                                  * (Optional) 
                                                                  */

#define EAPI_ID_HWMON_CPU_TEMP            EAPI_UINT32_C(0x20000) /* 0.1 Kelvins */ 
#define EAPI_ID_HWMON_CHIPSET_TEMP        EAPI_UINT32_C(0x20001) /* 0.1 Kelvins */ 
#define EAPI_ID_HWMON_SYSTEM_TEMP         EAPI_UINT32_C(0x20002) /* 0.1 Kelvins */ 

#define EAPI_KELVINS_OFFSET 2731
#define EAPI_ENCODE_CELCIUS(Celsius) EAPI_UINT32_C((((Celsius)*10))+EAPI_KELVINS_OFFSET)
#define EAPI_DECODE_CELCIUS(Celsius) ((Celsius)- EAPI_KELVINS_OFFSET)/10

#define EAPI_ID_HWMON_VOLTAGE_VCORE       EAPI_UINT32_C(0x21004) /* millivolts */
#define EAPI_ID_HWMON_VOLTAGE_2V5         EAPI_UINT32_C(0x21008) /* millivolts */
#define EAPI_ID_HWMON_VOLTAGE_3V3         EAPI_UINT32_C(0x2100C) /* millivolts */
#define EAPI_ID_HWMON_VOLTAGE_VBAT        EAPI_UINT32_C(0x21010) /* millivolts */
#define EAPI_ID_HWMON_VOLTAGE_5V          EAPI_UINT32_C(0x21014) /* millivolts */
#define EAPI_ID_HWMON_VOLTAGE_5VSB        EAPI_UINT32_C(0x21018) /* millivolts */
#define EAPI_ID_HWMON_VOLTAGE_12V         EAPI_UINT32_C(0x2101C) /* millivolts */

#define EAPI_ID_HWMON_FAN_CPU             EAPI_UINT32_C(0x22000) /* RPM */
#define EAPI_ID_HWMON_FAN_SYSTEM          EAPI_UINT32_C(0x22001) /* RPM */

#define ASUS_ID_HWMON_VOLTAGE_VTT    EAPI_UINT32_C(0xA1000) /* millivolts */
#define ASUS_ID_HWMON_VOLTAGE_DCIN   EAPI_UINT32_C(0xA1001) /* millivolts */
#define ASUS_ID_HWMON_CURRENT_SYSTEM EAPI_UINT32_C(0xA3000) /* milliamperes */

/* 
 * EApiBoardGetValue
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * pValue==NULL                           | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiBoardGetValue(
    __IN  EApiId_t  Id      , /* Value Id */
    __OUT uint32_t *pValue    /* Return Value */
    );



/*
 *
 *      B A C K L I G H T
 *
 */
/* IDS */
#define EAPI_ID_BACKLIGHT_1           EAPI_UINT32_C(0)
#define EAPI_ID_BACKLIGHT_2           EAPI_UINT32_C(1)
#define EAPI_ID_BACKLIGHT_3           EAPI_UINT32_C(2)

/* Backlight Values */
#define EAPI_BACKLIGHT_SET_ON         EAPI_UINT32_C(0)
#define EAPI_BACKLIGHT_SET_OFF        EAPI_UINT32_C(0xFFFFFFFF)
#define EAPI_BACKLIGHT_SET_DIMEST     EAPI_UINT32_C(0)
#define EAPI_BACKLIGHT_SET_BRIGHTEST  EAPI_UINT32_C(255)
/* 
 * EApiVgaGetBacklightEnable
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * pEnable==NULL                          | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE
EApiVgaGetBacklightEnable( 
    __IN  EApiId_t  Id      , /* Backlight Id */
    __OUT uint32_t *pEnable   /* Backlight Enable */
    );
/* 
 * EApiVgaSetBacklightEnable
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * Enable!=EAPI_BACKLIGHT_SET_ON&&
 * Enable!=EAPI_BACKLIGHT_SET_OFF         | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE
EApiVgaSetBacklightEnable(
    __IN  EApiId_t  Id      , /* Backlight Id */
    __IN  uint32_t  Enable    /* Backlight Enable */
    );
/* 
 * EApiVgaGetBacklightBrightness
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * pBright==NULL                          | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE
EApiVgaGetBacklightBrightness( 
    __IN  EApiId_t  Id      , /* Backlight Id */
    __OUT uint32_t *pBright   /* Backlight Brightness */
    );
/* 
 * EApiVgaSetBacklightBrightness
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * Bright>EAPI_BACKLIGHT_SET_BRIGHTEST    | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE
EApiVgaSetBacklightBrightness(
    __IN  EApiId_t  Id      , /* Backlight Id */
    __IN  uint32_t  Bright    /* Backlight Brightness */
    );


/*
 *
 *      S T O R A G E
 *
 */
/* IDs */
#define EAPI_ID_STORAGE_STD              EAPI_UINT32_C(0)
/* Dummy Example */
#define EAPI_PMG_ID_STORAGE_SAMPLE       EAPI_CREATE_CUST_ID('P', 'M', 'G', 0) 
/* 
 * EApiStorageCap
 * 
 * Condition                                    | Return Values 
 * ---------------------------------------------+------------------------------
 * Library Uninitialized                        | EAPI_STATUS_NOT_INITIALIZED
 * ((pStorageSize==NULL)&&(pBlockLength==NULL)) | EAPI_STATUS_INVALID_PARAMETER
 * Unsupported Id                               | EAPI_STATUS_UNSUPPORTED
 * Common Error                                 | Common Error Code
 * Else                                         | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiStorageCap(
    __IN  EApiId_t  Id            , /* Storage Area Id */
    __OUT uint32_t  *pStorageSize , /* Total */
    __OUT uint32_t  *pBlockLength   /* Write Block Length
                                     * & Alignment 
                                     */
    );
/* 
 * EApiStorageAreaRead
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * pBuffer==NULL                          | EAPI_STATUS_INVALID_PARAMETER
 * ByteCnt==0                             | EAPI_STATUS_INVALID_PARAMETER
 * BufLen==0                              | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Offset+ByteCnt>pStorageSize            | EAPI_STATUS_INVALID_BLOCK_LENGTH
 * Read Error                             | EAPI_STATUS_READ_ERROR
 * ByteCnt>BufLen                         | EAPI_STATUS_MORE_DATA
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiStorageAreaRead(
    __IN  EApiId_t  Id      , /* Storage Area Id */
    __IN  uint32_t  Offset  , /* Byte Offset */
    __OUT     void *pBuffer , /* Pointer to Data pBuffer */
    __IN  uint32_t  BufLen  , /* Data pBuffer Size in 
                               * bytes 
                               */
    __IN  uint32_t  ByteCnt   /* Number of bytes to read */
    );
/* 
 * EApiStorageAreaWrite
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * pBuffer==NULL                          | EAPI_STATUS_INVALID_PARAMETER
 * ByteCnt==0                             | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Offset%pBlockLength                    | EAPI_STATUS_INVALID_BLOCK_ALIGNMENT
 * ByteCnt%pBlockLength                   | EAPI_STATUS_INVALID_BLOCK_ALIGNMENT
 * Offset+ByteCnt>pStorageSize            | EAPI_STATUS_INVALID_BLOCK_LENGTH
 * Write Error                            | EAPI_STATUS_WRITE_ERROR
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiStorageAreaWrite(
    __IN  EApiId_t  Id      , /* Storage Area Id */
    __IN  uint32_t  Offset  , /* Byte Offset */
    __IN      void *pBuffer , /* Pointer to Data pBuffer */
    __IN  uint32_t  ByteCnt   /* Number of bytes to write */
    );


/*
 *
 *      I 2 C
 *
 */
/* IDs */
#define EAPI_ID_I2C_EXTERNAL       EAPI_UINT32_C(0) /* Baseboard I2C Interface
                                                     * required 
                                                     */
#define EAPI_ID_I2C_LVDS_1         EAPI_UINT32_C(1) /* LVDS/EDP 1 Interface 
                                                     * (optional)
                                                     */
#define EAPI_ID_I2C_LVDS_2         EAPI_UINT32_C(2) /* LVDS/EDP 2 Interface 
                                                     * (optional)
                                                     */

/*
 * I2C Address Format 
 *
 *  L = Set to 0   
 *  H = Set to 1   
 *  X = Don't Care(Direction Bit)
 *  0-F Address Bit 
 *
 *  Bits 31-16 are Reserved and should be set to 0
 *
 *  Bit Offset      | F E D C B A 9 8 7 6 5 4 3 2 1 0
 *  ----------------+--------------------------------
 *  7  Bit Address  | L L L L L L L L 6 5 4 3 2 1 0 X
 *  10 Bit Address  | H H H H L 9 8 X 7 6 5 4 3 2 1 0
 *
 *  Examples where Don't Care bits set to 0
 *             Encoded Encoded
 *  Address     7Bit    10Bit
 *    0x01      0x02   0xF001
 *    0x58      0xA0   0xF058
 *    0x59      0xA2   0xF059
 *    0x77      0xEE   0xF077
 *    0x3FF            0xF6FF
 *
 */
#define EAPI_I2C_ENC_7BIT_ADDR(x)  EAPI_UINT8_C(((x)&0x07F)<<1)
#define EAPI_I2C_DEC_7BIT_ADDR(x)  EAPI_UINT8_C(((x)>>1)&0x07F)
/* 
 * EApiI2CGetBusCap
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * pMaxBlkLen==NULL                       | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                             | EAPI_STATUS_UNSUPPORTED
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE
EApiI2CGetBusCap(
    __IN  EApiId_t  Id         , /* I2C Bus Id */
    __OUT uint32_t *pMaxBlkLen   /* Max Block Length 
                                  * Supported on this
                                  * interface 
                                  */
    );

/* 
 * EApiI2CWriteRead
 * 
 * Condition                                | Return Values 
 * -----------------------------------------+------------------------------
 * Library Uninitialized                    | EAPI_STATUS_NOT_INITIALIZED
 * (WriteBCnt>1)&&(pWBuffer==NULL)          | EAPI_STATUS_INVALID_PARAMETER
 * (ReadBCnt>1)&&(pRBuffer==NULL)           | EAPI_STATUS_INVALID_PARAMETER
 * (ReadBCnt>1)&&(RBufLen==0)               | EAPI_STATUS_INVALID_PARAMETER
 * ((WriteBCnt==0)&&(ReadBCnt==0))          | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                               | EAPI_STATUS_UNSUPPORTED
 * WriteBCnt>(pMaxBlkLen+1)                 | EAPI_STATUS_INVALID_BLOCK_LENGTH
 * ReadBCnt>(pMaxBlkLen+1)                  | EAPI_STATUS_INVALID_BLOCK_LENGTH
 * Bus Busy  SDA/SDC low                    | EAPI_STATUS_BUSY_COLLISION
 * Arbitration Error/Collision Error        | EAPI_STATUS_BUSY_COLLISION
 *   On Write 1 write cycle                 |
 *   SDA Remains low                        |
 * Timeout due to clock stretching          | EAPI_STATUS_TIMEOUT
 * start<Addr Byte><W>Nak                   | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte><R>Nak                   | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte><W>Ack<Data Byte 1>Nak   | EAPI_STATUS_WRITE_ERROR
 * ReadBCnt>(RBufLen+1)                     | EAPI_STATUS_MORE_DATA
 * Common Error                             | Common Error Code
 * Else                                     | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiI2CWriteReadRaw(
    __IN     EApiId_t  Id       , /* I2C Bus Id */
    __IN     uint8_t   Addr     , /* Encoded 7Bit I2C
                                   * Device Address 
                                   */
    __INOPT  void     *pWBuffer , /* Write Data pBuffer */
    __IN     uint32_t  WriteBCnt, /* Number of Bytes to 
                                   * write plus 1
                                   */
    __OUTOPT void     *pRBuffer , /* Read Data pBuffer */
    __IN     uint32_t  RBufLen  , /* Data pBuffer Length */
    __IN     uint32_t  ReadBCnt   /* Number of Bytes to 
                                   * Read plus 1
                                   */
    );

#define EApiI2CWriteRaw(Id, Addr, pBuffer, ByteCnt)         \
          EApiI2CWriteReadRaw(Id, Addr, pBuffer, ByteCnt, NULL, 0, 0)
#define EApiI2CReadRaw(Id, Addr, pBuffer, BufLen, ByteCnt) \
          EApiI2CWriteReadRaw(Id, Addr, NULL, 0, pBuffer, BufLen, ByteCnt)


#define EAPI_I2C_ENC_10BIT_ADDR(x) EAPI_UINT32_C(((x)&0xFF)|(((x)&0x0300)<<1)|0xF000)
#define EAPI_I2C_DEC_10BIT_ADDR(x) EAPI_UINT32_C(((x)&0xFF)|(((x)>>1)&0x300))
#define EAPI_I2C_IS_10BIT_ADDR(x)  (((x)&0xF800)==0xF000)
#define EAPI_I2C_IS_7BIT_ADDR(x)   (!EAPI_I2C_IS_10BIT_ADDR(x))

/*
 * I2C Transfer Types
 * Bits 31 & 30 Selects Command Type
 *
 * Transfer Type 1: 
 * Address Format : 7Bit
 * Command Type   : None
 * Data Direction : Write
 * Start<Addr Byte><W>Ack<Data Byte 1>Ack Stop
 *
 * Transfer Type 2: 
 * Address Format : 7Bit
 * Command Type   : None
 * Data Direction : Read 
 * Start<Addr Byte><R>Ack<Data Byte 1>Nak Stop
 *
 * Transfer Type 3: 
 * Address Format : 7Bit
 * Command Type   : Standard
 * Data Direction : Write
 * Start<Addr Byte><W>Ack<CMD Byte>Ack<Data Byte 1>Ack Stop
 *
 * Transfer Type 4: 
 * Address Format : 7Bit
 * Command Type   : Standard
 * Data Direction : Read
 * Start<Addr Byte><W>Ack<CMD Byte>Ack 
 * Start<Addr Byte><R>Ack<Data Byte 1>Nak Stop
 *
 * Transfer Type 5: 
 * Address Format : 7Bit
 * Command Type   : Extended
 * Data Direction : Write
 * Start<Addr Byte><W>Ack<CMD Byte MSB>Ack<CMD Byte LSB>Ack<Data Byte 1>Ack Stop
 *
 * Transfer Type 6: 
 * Address Format : 7Bit
 * Command Type   : Extended
 * Data Direction : Read
 * Start<Addr Byte><W>Ack<CMD Byte MSB>Ack<CMD Byte LSB>Ack 
 * Start<Addr Byte><R>Ack<Data Byte 1>Nak Stop
 *
 * Transfer Type 7: 
 * Address Format : 10Bit
 * Command Type   : None
 * Data Direction : Write
 * Start<Addr Byte MSB><W>Ack<Addr Byte LSB>Ack<Data Byte 1>Ack Stop
 *
 * Transfer Type 8: 
 * Address Format : 10Bit
 * Command Type   : None
 * Data Direction : Read
 * Start<Addr Byte MSB><W>Ack<Addr Byte LSB>Ack 
 * Start<Addr Byte MSB><R>Ack<Data Byte 1>Nak Stop
 *
 * Transfer Type 9: 
 * Address Format : 10Bit
 * Command Type   : Standard
 * Data Direction : Write
 * Start<Addr Byte MSB><W>Ack<Addr Byte LSB>Ack<CMD Byte>Ack<Data Byte 1>Ack Stop
 *
 * Transfer Type 10: 
 * Address Format : 10Bit
 * Command Type   : Standard
 * Data Direction : Read
 * Start<Addr Byte MSB><W>Ack<Addr Byte LSB>Ack<CMD Byte>Ack 
 * Start<Addr Byte MSB><R>Ack<Data Byte 1>Nak Stop
 *
 * Transfer Type 11: 
 * Address Format : 10Bit
 * Command Type   : Extended
 * Data Direction : Write
 * Start<Addr Byte MSB><W>Ack<Addr Byte LSB>Ack<CMD Byte MSB>Ack<CMD Byte LSB>Ack<Data Byte 1>Ack Stop
 *
 * Transfer Type 12: 
 * Address Format : 10Bit
 * Command Type   : Extended
 * Data Direction : Read
 * Start<Addr Byte MSB><W>Ack<Addr Byte LSB>Ack<CMD Byte MSB>Ack<CMD Byte LSB>Ack
 * Start<Addr Byte MSB><R>Ack<Data Byte 1>Nak Stop
 *
 */
#define EAPI_I2C_STD_CMD          EAPI_UINT32_C(0<<30)
#define EAPI_I2C_EXT_CMD          EAPI_UINT32_C(2<<30)
#define EAPI_I2C_NO_CMD           EAPI_UINT32_C(1<<30)    
#define EAPI_I2C_CMD_TYPE_MASK    EAPI_UINT32_C(3<<30)    

#define EAPI_I2C_ENC_STD_CMD(x)  EAPI_UINT32_C(((x)&0xFF)|EAPI_I2C_STD_CMD)
#define EAPI_I2C_ENC_EXT_CMD(x)  EAPI_UINT32_C(((x)&0xFFFF)|EAPI_I2C_EXT_CMD)
#define EAPI_I2C_IS_EXT_CMD(x)   (EAPI_UINT32_C((x)&(EAPI_I2C_CMD_TYPE_MASK))==EAPI_I2C_EXT_CMD)
#define EAPI_I2C_IS_STD_CMD(x)   (EAPI_UINT32_C((x)&(EAPI_I2C_CMD_TYPE_MASK))==EAPI_I2C_STD_CMD)
#define EAPI_I2C_IS_NO_CMD(x)    (EAPI_UINT32_C((x)&(EAPI_I2C_CMD_TYPE_MASK))==EAPI_I2C_NO_CMD)
/* 
 * EApiI2CReadTransfer
 * Addr Byte 1 Below Designates Addr MSB in a 10bit address transfer and 
 * the complete address in an 7bit address transfer.
 * 
 * Condition                                | Return Values 
 * -----------------------------------------+------------------------------
 * Library Uninitialized                    | EAPI_STATUS_NOT_INITIALIZED
 * pBuffer==NULL                            | EAPI_STATUS_INVALID_PARAMETER
 * BufLen==0                                | EAPI_STATUS_INVALID_PARAMETER
 * ByteCnt==0                               | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                               | EAPI_STATUS_UNSUPPORTED
 * ByteCnt>pMaxBlkLen                       | EAPI_STATUS_INVALID_BLOCK_LENGTH
 * Bus Busy  SDA/SDC low                    | EAPI_STATUS_BUSY_COLLISION
 * Arbitration Error/Collision Error        | EAPI_STATUS_BUSY_COLLISION
 *   On Write 1 write cycle                 |
 *   SDA Remains low                        |
 * Time-out due to clock stretching         | EAPI_STATUS_TIMEOUT
 * start<Addr Byte 1><W>Nak                 | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte 1><R>Nak                 | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte 1><W>Ack<Addr Byte 2>Nak | EAPI_STATUS_WRITE_ERROR or
 *                                          | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte 1><W>Ack<CMD Byte 1>Nak  | EAPI_STATUS_WRITE_ERROR
 * start<Addr Byte 1><W>Ack<Data Byte 1>Nak | EAPI_STATUS_WRITE_ERROR
 * ByteCnt>BufLen                           | EAPI_STATUS_MORE_DATA
 * Common Error                             | Common Error Code
 * Else                                     | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiI2CReadTransfer(
    __IN  EApiId_t  Id      , /* I2C Bus Id */
    __IN  uint32_t  Addr    , /* Encoded 7/10Bit I2C
                               * Device Address
                               */
    __IN  uint32_t  Cmd     , /* I2C Command/Offset */
    __OUT     void *pBuffer , /* Transfer Data pBuffer */
    __IN  uint32_t  BufLen  , /* Data pBuffer Length */
    __IN  uint32_t  ByteCnt   /* Byte Count to read */
    );
/* 
 * EApiI2CWriteTransfer
 * Addr Byte 1 Below Designates Addr MSB in a 10bit address transfer and 
 * the complete address in an 7bit address transfer.
 * 
 * Condition                                | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                    | EAPI_STATUS_NOT_INITIALIZED
 * pBuffer==NULL                            | EAPI_STATUS_INVALID_PARAMETER
 * ByteCnt==0                               | EAPI_STATUS_INVALID_PARAMETER
 * unknown Id                               | EAPI_STATUS_UNSUPPORTED
 * ByteCnt+(overhead)>pMaxBlkLen            | EAPI_STATUS_INVALID_BLOCK_LENGTH
 * Bus Busy  SDA/SDC low                    | EAPI_STATUS_BUSY_COLLISION
 * Arbitration Error/Collision Error        | EAPI_STATUS_BUSY_COLLISION
 *   On Write 1 write cycle                 |
 *   SDA Remains low                        |
 * Time-out due to clock stretching         | EAPI_STATUS_TIMEOUT
 * start<Addr Byte 1><W>Nak                 | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte 1><R>Nak                 | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte 1><W>Ack<Addr Byte 2>Nak | EAPI_STATUS_WRITE_ERROR or
 *                                          | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte 1><W>Ack<CMD Byte 1>Nak  | EAPI_STATUS_WRITE_ERROR
 * start<Addr Byte 1><W>Ack<Data Byte 1>Nak | EAPI_STATUS_WRITE_ERROR
 * Common Error                             | Common Error Code
 * Else                                     | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE
EApiI2CWriteTransfer(
    __IN  EApiId_t  Id      , /* I2C Bus Id */
    __IN  uint32_t  Addr    , /* Encoded 7/10Bit I2C 
                               * Device Address 
                               */
    __IN  uint32_t  Cmd     , /* I2C Command/Offset */
    __IN      void *pBuffer , /* Transfer Data pBuffer */
    __IN  uint32_t  ByteCnt   /* Byte Count to write */
    );

/*
 * I2C Probe Types
 *
 * Probe Type 1: 
 * Address Format : 7Bit
 * Start<Addr Byte><W>Ack Stop
 *
 * Probe Type 2: 
 * Address Format : 10Bit
 * Start<Addr Byte MSB><W>Ack<Addr Byte LSB>Ack Stop
 *
 */

/* 
 * EApiI2CProbeDevice
 * 
 * Condition                                    | Return Values 
 * ---------------------------------------------+------------------------------
 * Library Uninitialized                        | EAPI_STATUS_NOT_INITIALIZED
 * Bus Busy  SDA/SDC low                        | EAPI_STATUS_BUSY_COLLISION
 * Arbitration Error/Collision Error            | EAPI_STATUS_BUSY_COLLISION
 *   On Write 1 write cycle                     |
 *   SDA Remains low                            |
 * Time-out due to clock stretching             | EAPI_STATUS_TIMEOUT
 *                                              |
 * 7Bit Address                                 |
 * start<Addr Byte><W>Nak                       | EAPI_STATUS_NOT_FOUND
 *                                              |
 * 10Bit Address                                |
 * start<Addr Byte MSB><W>Nak                   | EAPI_STATUS_NOT_FOUND
 * start<Addr Byte MSB><W>Ack<Addr Byte LSB>Nak | EAPI_STATUS_NOT_FOUND
 *                                              |
 * Common Error                                 | Common Error Code
 * Else                                         | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE
EApiI2CProbeDevice(
    __IN  EApiId_t  Id   , /* I2C Bus Id */
    __IN  uint32_t  Addr   /* Encoded 7/10Bit 
                            * I2C Device Address 
                            */
    );

/*
 *
 *      W A T C H D O G
 *
 */

/*
 *
 * After EApiWDogStart
 *
 * |<- Delay ->|<- Event Timeout ->|<- Reset Timeout ->|
 * A-----------B-------------------C-------------------D
 *
 *
 * After EApiWDogTrigger
 *
 * |<- Event Timeout ->|<- Reset Timeout ->|
 * E-------------------F-------------------G
 *
 *
 * Stage A
 *  Watchdog is started.
 *
 * Stage B
 *  Initial Delay Period is exhausted.
 *
 * Stage C/F
 *  Event is triggered, NMI, IRQ, or PIN is Triggered.
 *  To Allow for possible Software Recovery.
 *
 * Stage D/G
 *  System is reset.
 *
 * Stage E
 *  Watchdog is Triggered.
 *
 * EApiWDogTrigger/EApiWDogStop Must be called before Stage C/F
 * to prevent event from being generated.
 *
 * EApiWDogTrigger/EApiWDogStop Must be called before Stage D/G
 * to prevent The system from being reset.
 *
 */

/* 
 * EApiWDogGetCap
 * 
 * Condition                              | Return Values 
 * ---------------------------------------+------------------------------
 * Library Uninitialized                  | EAPI_STATUS_NOT_INITIALIZED
 * Unsupported                            | EAPI_STATUS_UNSUPPORTED
 * pMaxDelay==NULL&&                      |
 * pMaxResetTimeout==NULL&&               |
 * pMaxEventTimeout==NULL                 | EAPI_STATUS_INVALID_PARAMETER
 * Common Error                           | Common Error Code
 * Else                                   | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE
EApiWDogGetCap(
    __OUTOPT uint32_t *pMaxDelay       , /* Maximum Supported 
                                         * Delay in milliseconds
                                         */
    __OUTOPT uint32_t *pMaxEventTimeout, /* Maximum Supported 
                                         * Event Timeout in 
                                         * milliseconds
                                         * 0 == Unsupported
                                         */
    __OUTOPT uint32_t *pMaxResetTimeout  /* Maximum Supported 
                                         * Reset Timeout in 
                                         * milliseconds
                                         */
    );
/* 
 * EApiWDogStart
 * 
 * Condition                        | Return Values 
 * ---------------------------------+------------------------------
 * Library Uninitialized            | EAPI_STATUS_NOT_INITIALIZED
 * Unsupported                      | EAPI_STATUS_UNSUPPORTED
 * Delay>pMaxDelay                  | EAPI_STATUS_INVALID_PARAMETER
 * EventTimeout>pMaxEventTimeout    | EAPI_STATUS_INVALID_PARAMETER
 * ResetTimeout>pMaxResetTimeout    | EAPI_STATUS_INVALID_PARAMETER
 * Already Running                  | EAPI_STATUS_RUNNING
 * Common Error                     | Common Error Code
 * Else                             | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiWDogStart(
    __IN  uint32_t Delay       , /* Delay in milliseconds */
    __IN  uint32_t EventTimeout, /* Event Timeout in 
                                  * milliseconds 
                                  */
    __IN  uint32_t ResetTimeout  /* Reset Timeout in 
                                  * milliseconds 
                                  */
    );

/* 
 * EApiWDogTrigger
 * 
 * Condition                        | Return Values 
 * ---------------------------------+------------------------------
 * Library Uninitialized            | EAPI_STATUS_NOT_INITIALIZED
 * Unsupported                      | EAPI_STATUS_UNSUPPORTED
 * Watchdog Not Started             | EAPI_STATUS_ERROR
 * Common Error                     | Common Error Code
 * Else                             | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiWDogTrigger(void);

/* 
 * EApiWDogStop
 * 
 * Condition                        | Return Values 
 * ---------------------------------+------------------------------
 * Library Uninitialized            | EAPI_STATUS_NOT_INITIALIZED
 * Unsupported                      | EAPI_STATUS_UNSUPPORTED
 * Common Error                     | Common Error Code
 * Else                             | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiWDogStop(void);


/*
 *
 *      G P I O
 *
 */
/*
 *
 * +-----------------------------------------------+
 * |              Physical GPIO                    |
 * +-----+-----+-----+-----+-----+-----+-----+-----+
 * |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 * +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |     |     |     |     |     |     |
 *    |     |     |     |     |     |     |     +-----------------------------------------------+
 *    |     |     |     |     |     |     |     |                                               |
 *    |     |     |     |     |     |     +-----|-----------------------------------------+     |
 *    |     |     |     |     |     |     |     |                                         |     |
 *    |     |     |     |     |     +-----|-----|-----------------------------------+     |     |
 *    |     |     |     |     |     |     |     |                                   |     |     |
 *    |     |     |     |     +-----|-----|-----|-----------------------------+     |     |     |
 *    |     |     |     |     |     |     |     |                             |     |     |     |
 *    |     |     |     +-----|-----|-----|-----|-----------------------+     |     |     |     |
 *    |     |     |     |     |     |     |     |                       |     |     |     |     |
 *    |     |     +-----|-----|-----|-----|-----|-----------------+     |     |     |     |     |
 *    |     |     |     |     |     |     |     |                 |     |     |     |     |     |
 *    |     +-----|-----|-----|-----|-----|-----|-----------+     |     |     |     |     |     |
 *    |     |     |     |     |     |     |     |           |     |     |     |     |     |     |
 *    +-----|-----|-----|-----|-----|-----|-----|-----+     |     |     |     |     |     |     |
 *    |     |     |     |     |     |     |     |     |     |     |     |     |     |     |     |
 *    |     |     |     |     |     |     |     |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |     |     |     |     |     |     |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |     |     |     |     |     |     |     |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |     |     |     |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |     |     |     |  |            EAPI_ID_GPIO_BITMASK00             |
 *    |     |     |     |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |     |     |     |
 *    |     |     |     |     |     |     |     |     0     0     0     0     0     0     0
 *    |     |     |     |     |     |     |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |     |     |     |     |     |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |     |     |     |     |     |     |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |     |     |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |     |     |  |            EAPI_ID_GPIO_GPIO07                |
 *    |     |     |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |     |     |
 *    |     |     |     |     |     |     |     0     0     0     0     0     0     0
 *    |     |     |     |     |     |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |     |     |     |     |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |     |     |     |     |     |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |     |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |     |  |            EAPI_ID_GPIO_GPIO06                |
 *    |     |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |     |
 *    |     |     |     |     |     |     0     0     0     0     0     0     0
 *    |     |     |     |     |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |     |     |     |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |     |     |     |     |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |  |            EAPI_ID_GPIO_GPIO05                |
 *    |     |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |     |
 *    |     |     |     |     |     0     0     0     0     0     0     0
 *    |     |     |     |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |     |     |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |     |     |     |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |  |            EAPI_ID_GPIO_GPIO04                |
 *    |     |     |     |  +-----------------------------------------------+
 *    |     |     |     |
 *    |     |     |     |     0     0     0     0     0     0     0
 *    |     |     |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |     |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |     |     |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |     |     |  +-----------------------------------------------+
 *    |     |     |  |            EAPI_ID_GPIO_GPIO03                |
 *    |     |     |  +-----------------------------------------------+
 *    |     |     |
 *    |     |     |     0     0     0     0     0     0     0
 *    |     |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |     |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |     |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |     |  +-----------------------------------------------+
 *    |     |  |            EAPI_ID_GPIO_GPIO02                |
 *    |     |  +-----------------------------------------------+
 *    |     |
 *    |     |     0     0     0     0     0     0     0
 *    |  +-----+-----+-----+-----+-----+-----+-----+-----+
 *    |  | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 *    |  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 *    |  +-----------------------------------------------+
 *    |  |            EAPI_ID_GPIO_GPIO01                |
 *    |  +-----------------------------------------------+
 *    |
 *    |     0     0     0     0     0     0     0
 * +-----+-----+-----+-----+-----+-----+-----+-----+
 * | Bit | Bit | Bit | Bit | Bit | Bit | Bit | Bit |
 * |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |
 * +-----------------------------------------------+
 * |            EAPI_ID_GPIO_GPIO00                |
 * +-----------------------------------------------+
 */
/* IDs */
/*
 * Individual ID Per GPIO Mapping
 */
#define EAPI_GPIO_GPIO_ID(GPIO_NUM)    EAPI_UINT32_C(GPIO_NUM)
#define EAPI_GPIO_GPIO_BITMASK  EAPI_UINT32_C(1)


#define EAPI_ID_GPIO_GPIO00     EAPI_GPIO_GPIO_ID(0) /* (Optional) */
#define EAPI_ID_GPIO_GPIO01     EAPI_GPIO_GPIO_ID(1) /* (Optional) */
#define EAPI_ID_GPIO_GPIO02     EAPI_GPIO_GPIO_ID(2) /* (Optional) */
#define EAPI_ID_GPIO_GPIO03     EAPI_GPIO_GPIO_ID(3) /* (Optional) */
#define EAPI_ID_GPIO_GPIO04     EAPI_GPIO_GPIO_ID(4) /* (Optional) */
#define EAPI_ID_GPIO_GPIO05     EAPI_GPIO_GPIO_ID(5) /* (Optional) */
#define EAPI_ID_GPIO_GPIO06     EAPI_GPIO_GPIO_ID(6) /* (Optional) */
#define EAPI_ID_GPIO_GPIO07     EAPI_GPIO_GPIO_ID(7) /* (Optional) */

/*
 * Multiple GPIOs Per ID Mapping
 */
#define EAPI_GPIO_BANK_ID(GPIO_NUM)     EAPI_UINT32_C(0x10000|((GPIO_NUM)>>5))
#define EAPI_GPIO_BANK_MASK(GPIO_NUM) EAPI_UINT32_C((1<<((GPIO_NUM)&0x1F))
#define EAPI_GPIO_BANK_TEST_STATE(GPIO_NUM, TState, TValue) \
                        (((TValue>>((GPIO_NUM)&0x1F))&1)==(TState))

#define EAPI_ID_GPIO_BANK00    EAPI_GPIO_BANK_ID( 0) /* GPIOs  0 - 31 
                                                      * (optional)
                                                      */
#define EAPI_ID_GPIO_BANK01    EAPI_GPIO_BANK_ID(32) /* GPIOs 32 - 63 
                                                      * (optional)
                                                      */
#define EAPI_ID_GPIO_BANK02    EAPI_GPIO_BANK_ID(64) /* GPIOs 64 - 95
                                                      * (optional)
                                                      */


/* Bit mask Bit States */
#define EAPI_GPIO_BITMASK_SELECT   1
#define EAPI_GPIO_BITMASK_NOSELECT 0

/* Levels */
#define EAPI_GPIO_LOW     0
#define EAPI_GPIO_HIGH    1

/* Directions */
#define EAPI_GPIO_INPUT   1
#define EAPI_GPIO_OUTPUT  0

/* 
 * EApiGPIOGetDirectionCaps
 * 
 * Condition                          | Return Values 
 * -----------------------------------+------------------------------
 * Library Uninitialized              | EAPI_STATUS_NOT_INITIALIZED
 * ((pOutputs==NULL)&&(pInputs==NULL))| EAPI_STATUS_INVALID_PARAMETER
 * Unsupported Id                     | EAPI_STATUS_UNSUPPORTED
 * Common Error                       | Common Error Code
 * Else                               | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE 
EApiGPIOGetDirectionCaps(
    __IN     EApiId_t Id        , /* GPIO Id */
    __OUTOPT uint32_t *pInputs  , /* Supported GPIO Input
                                   * Bit Mask 
                                   */
    __OUTOPT uint32_t *pOutputs   /* Supported GPIO Output
                                   * Bit Mask 
                                   */
    );
/* 
 * EApiGPIOGetDirection
 * 
 * Condition                          | Return Values 
 * -----------------------------------+------------------------------
 * Library Uninitialized              | EAPI_STATUS_NOT_INITIALIZED
 * Bitmask==0                         | EAPI_STATUS_INVALID_PARAMETER
 * Unsupported Id                     | EAPI_STATUS_UNSUPPORTED
 * (Bitmask&~(pInputs|pOutputs))      | EAPI_STATUS_INVALID_BITMASK
 * Common Error                       | Common Error Code
 * Else                               | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiGPIOGetDirection(
    __IN  EApiId_t Id          , /* GPIO Id */
    __IN  uint32_t Bitmask     , /* Bit mask of Affected
                                  * Bits 
                                  */
    __OUT uint32_t *pDirection   /* Current Direction */
    );
/* 
 * EApiGPIOSetDirection
 * 
 * Condition                          | Return Values 
 * -----------------------------------+------------------------------
 * Library Uninitialized              | EAPI_STATUS_NOT_INITIALIZED
 * Bitmask==0                         | EAPI_STATUS_INVALID_PARAMETER
 * Unsupported Id                     | EAPI_STATUS_UNSUPPORTED
 * (Bitmask&~(pInputs|pOutputs))      | EAPI_STATUS_INVALID_BITMASK
 * (Bitmask&pDirection)&~pInputs      | EAPI_STATUS_INVALID_DIRECTION
 * (Bitmask&~pDirection)&~pOutputs    | EAPI_STATUS_INVALID_DIRECTION
 * Common Error                       | Common Error Code
 * Else                               | EAPI_STATUS_SUCCESS
 */
EApiStatus_t 
EAPI_CALLTYPE 
EApiGPIOSetDirection(
    __IN  EApiId_t Id          , /* GPIO Id */
    __IN  uint32_t Bitmask     , /* Bit mask of Affected 
                                  * Bits 
                                  */
    __IN  uint32_t Direction     /* Direction */
    );
/* 
 * EApiGPIOGetLevel
 * 
 * Condition                          | Return Values 
 * -----------------------------------+------------------------------
 * Library Uninitialized              | EAPI_STATUS_NOT_INITIALIZED
 * Bitmask==0                         | EAPI_STATUS_INVALID_PARAMETER
 * Unsupported Id                     | EAPI_STATUS_UNSUPPORTED
 * (Bitmask&~(pInputs|pOutputs))      | EAPI_STATUS_INVALID_BITMASK
 * Common Error                       | Common Error Code
 * Else                               | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE 
EApiGPIOGetLevel(
    __IN  EApiId_t Id          , /* GPIO Id */
    __IN  uint32_t Bitmask     , /* Bit mask of Affected
                                  * Bits 
                                  */
    __OUT uint32_t *pLevel       /* Current Level */
    );
/* 
 * EApiGPIOSetLevel
 * 
 * Condition                          | Return Values 
 * -----------------------------------+------------------------------
 * Library Uninitialized              | EAPI_STATUS_NOT_INITIALIZED
 * Bitmask==0                         | EAPI_STATUS_INVALID_PARAMETER
 * Unsupported Id                     | EAPI_STATUS_UNSUPPORTED
 * Common Error                       | Common Error Code
 * Else                               | EAPI_STATUS_SUCCESS
 */
EApiStatus_t
EAPI_CALLTYPE 
EApiGPIOSetLevel(
    __IN  EApiId_t Id          , /* GPIO Id */
    __IN  uint32_t Bitmask     , /* Bit mask of Affected 
                                  * Bits 
                                  */
    __IN  uint32_t Level         /* Level */  
    );

/*
 *
 * ASUS Customize Function
 *
*/
AsusStatus_t 
ASUS_CALLTYPE 
AsusWDogStartWdtService(
    __IN  uint32_t Delay       , /* Delay in milliseconds */
    __IN  uint32_t ResetTimeout  /* Reset Timeout in 
                                  * milliseconds 
                                  */
    );

#define ASUS_ID_SYSTEM_BOOT_BOOTUP         EAPI_UINT32_C(0)
#define ASUS_ID_SYSTEM_BOOT_SHUTDOWN       EAPI_UINT32_C(1)
#define ASUS_ID_SYSTEM_BOOT_SLEEP          EAPI_UINT32_C(2)
#define ASUS_ID_SYSTEM_BOOT_RESTART        EAPI_UINT32_C(3)

#define ASUS_SYSTEM_BOOT_DISABLE_FREQUENCY               EAPI_UINT32_C(0)
#define ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_DAILY          EAPI_UINT32_C(1)
#define ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_WEEKLY         EAPI_UINT32_C(2)
#define ASUS_SYSTEM_BOOT_ENABLE_FREQUENCY_MONTHLY        EAPI_UINT32_C(3)

#define ASUS_SYSTEM_BOOT_WEEKLY_SUNDAY     EAPI_UINT32_C(0)
#define ASUS_SYSTEM_BOOT_WEEKLY_MONDAY     EAPI_UINT32_C(1)
#define ASUS_SYSTEM_BOOT_WEEKLY_TUESDAY    EAPI_UINT32_C(2)
#define ASUS_SYSTEM_BOOT_WEEKLY_WEDNESDAY  EAPI_UINT32_C(3)
#define ASUS_SYSTEM_BOOT_WEEKLY_THURSDAY   EAPI_UINT32_C(4)
#define ASUS_SYSTEM_BOOT_WEEKLY_FRIDAY     EAPI_UINT32_C(5)
#define ASUS_SYSTEM_BOOT_WEEKLY_SATURDAY   EAPI_UINT32_C(6)

AsusStatus_t 
ASUS_CALLTYPE 
AsusSystemBootSet(
    __IN uint32_t Id,
    __IN uint32_t Frequency,
    __IN uint32_t DayofMonth,
    __IN uint32_t DayofWeek,
    __IN uint32_t Hour,
    __IN uint32_t Minute,
    __IN uint32_t Second
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusSystemBootGet(
    __IN uint32_t Id,
    __OUT uint32_t *pFrequency,
    __OUT uint32_t *pDayofMonth,
    __OUT uint32_t *pDayofWeek,
    __OUT uint32_t *pHour,
    __OUT uint32_t *pMinute,
    __OUT uint32_t *pSecond
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusBoardSetValue(
    __IN EApiId_t Id,
    __IN uint32_t Value
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusLedTurnOn(
    __IN uint32_t LedId,
    __IN uint32_t Color
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusLedTurnOff(
    __IN uint32_t LedId
    );

#define LED_LIGHT_COLOR_BLUE EAPI_UINT32_C(0x00000001)
#define LED_LIGHT_COLOR_GREEN EAPI_UINT32_C(0x00000002)
#define LED_LIGHT_COLOR_RED EAPI_UINT32_C(0x00000004)

AsusStatus_t 
ASUS_CALLTYPE 
AsusLedGetNumberofLeds(__OUT uint32_t *pValue);

typedef struct LedInfo_s {
    uint32_t LedId;
    uint32_t SupportedColor;
    uint32_t SystemOccupied;
} LedInfo_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusLedGetInfo(
    __OUT struct LedInfo_s *pLedInfo,
    __INOUT uint32_t *pLedCnt
    );

#define LED_OCCUPIED_BY_SYSTEM EAPI_UINT32_C(0x00000001)
#define LED_OCCUPIED_BY_USER EAPI_UINT32_C(0x00000000)

AsusStatus_t 
ASUS_CALLTYPE 
AsusLedSetSystemOccupied(
    __IN uint32_t LedId,
    __IN uint32_t SystemOccupied
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetNumberofModems(__OUT uint32_t *pValue);

#define MAX_CHAR_LENGTH_MODEM_INFO 256
typedef struct ConnMgrModemInfo_s {
    char Index;
    char Path[MAX_CHAR_LENGTH_MODEM_INFO];
    char Manufacturer[MAX_CHAR_LENGTH_MODEM_INFO];
    char ModemName[MAX_CHAR_LENGTH_MODEM_INFO];
    char FirmwareRevision[MAX_CHAR_LENGTH_MODEM_INFO];
} ConnMgrModemInfo_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetModemInfo(
    __OUT struct ConnMgrModemInfo_s *pModemList,
    __INOUT uint32_t *pModemCnt
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemStartNetwork(__IN uint32_t Index);

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemStopNetwork(__IN uint32_t Index);

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemPowerOn(__IN uint32_t Index);

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemPowerOff(__IN uint32_t Index);

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemRestart(__IN uint32_t Index);

/* SIM Slot Id */
#define ASUS_CMM_SIM_SLOT_0     0
#define ASUS_CMM_SIM_SLOT_1     1

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSwitchSIM(
    __IN uint32_t Index,
    __IN uint32_t  SimSlotId
    );

enum ASUS_CMM_REGISTRATION_STATE {
    CMMODEM_STATE_FAILED = EAPI_UINT32_C(0xA4000), //The modem is unusable.
    CMMODEM_STATE_UNKNOWN, //State unknown or not reportable.
    CMMODEM_STATE_INITIALIZING, //The modem is currently being initialized.
    CMMODEM_STATE_LOCKED, //The modem needs to be unlocked.
    CMMODEM_STATE_DISABLED, //The modem is not enabled and is powered down.
    CMMODEM_STATE_DISABLING, //The modem is currently transitioning to the MM_MODEM_STATE_DISABLED state.
    CMMODEM_STATE_ENABLING, //The modem is currently transitioning to the MM_MODEM_STATE_ENABLED state.
    CMMODEM_STATE_ENABLED, //The modem is enabled and powered on but not registered with a network provider and not available for data connections.
    CMMODEM_STATE_SEARCHING, //The modem is searching for a network provider to register with.
    CMMODEM_STATE_REGISTERED, //The modem is registered with a network provider, and data connections and messaging may be available for use.
    CMMODEM_STATE_DISCONNECTING, //The modem is disconnecting and deactivating the last active packet data bearer. This state will not be entered if more than one packet data bearer is active and one of the active bearers is deactivated.
    CMMODEM_STATE_CONNECTING, //The modem is activating and connecting the first packet data bearer. Subsequent bearer activations when another bearer is already active do not cause this state to be entered.
    CMMODEM_STATE_CONNECTED //One or more packet data bearers is active and connected.
};

enum ASUS_CMM_RADIO_INTERFACE {
    CMMODEM_ACCESS_TECHNOLOGY_UNKNOWN = EAPI_UINT32_C(0xA5000), //The access technology used is unknown.
    CMMODEM_ACCESS_TECHNOLOGY_POTS, //Analog wireline telephone.
    CMMODEM_ACCESS_TECHNOLOGY_GSM, //GSM.
    CMMODEM_ACCESS_TECHNOLOGY_GSM_COMPACT, //Compact GSM.
    CMMODEM_ACCESS_TECHNOLOGY_GPRS, //GPRS.
    CMMODEM_ACCESS_TECHNOLOGY_EDGE, //EDGE (ETSI 27.007: "GSM w/EGPRS").
    CMMODEM_ACCESS_TECHNOLOGY_UMTS, //UMTS (ETSI 27.007: "UTRAN").
    CMMODEM_ACCESS_TECHNOLOGY_HSDPA, //HSDPA (ETSI 27.007: "UTRAN w/HSDPA").
    CMMODEM_ACCESS_TECHNOLOGY_HSUPA, //HSUPA (ETSI 27.007: "UTRAN w/HSUPA").
    CMMODEM_ACCESS_TECHNOLOGY_HSPA, //HSPA (ETSI 27.007: "UTRAN w/HSDPA and HSUPA").
    CMMODEM_ACCESS_TECHNOLOGY_HSPA_PLUS, //HSPA+ (ETSI 27.007: "UTRAN w/HSPA+").
    CMMODEM_ACCESS_TECHNOLOGY_1XRTT, //CDMA2000 1xRTT.
    CMMODEM_ACCESS_TECHNOLOGY_EVDO0, //CDMA2000 EVDO revision 0.
    CMMODEM_ACCESS_TECHNOLOGY_EVDOA, //CDMA2000 EVDO revision A.
    CMMODEM_ACCESS_TECHNOLOGY_EVDOB, //CDMA2000 EVDO revision B.
    CMMODEM_ACCESS_TECHNOLOGY_LTE, //LTE (ETSI 27.007: "E-UTRAN")
    CMMODEM_ACCESS_TECHNOLOGY_5GNR, //5GNR (ETSI 27.007: "NG-RAN"). Since 1.14.
    CMMODEM_ACCESS_TECHNOLOGY_ANY //Mask specifying all access technologies.
};

/* Flight Mode */
enum ASUS_CMM_FLIGHT_MODE_STATE {
    ASUS_CMM_FLIGHT_MODE_OFF = 0,
    ASUS_CMM_FLIGHT_MODE_ON
};

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSetFlightMode(
    __IN uint32_t Index,
    __IN enum ASUS_CMM_FLIGHT_MODE_STATE FlightMode
    );

typedef struct ConnMgrModemAttachedStatus_s {
    enum ASUS_CMM_REGISTRATION_STATE RegistrationState;
    enum ASUS_CMM_RADIO_INTERFACE RadioInterface;
    enum ASUS_CMM_FLIGHT_MODE_STATE FlightModeState;
} ConnMgrModemAttachedStatus_t;


AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetAttachedStatus(
    __IN uint32_t Index,
    __OUT struct ConnMgrModemAttachedStatus_s *pStatus
    );

typedef struct ConnMgrModemIpv4Status_s {
    __OUT char *pAddress;
    __INOUT uint32_t *pAddressLen;
    __OUT char *pGateway;
    __INOUT uint32_t *pGatewayLen;
    __OUT char *pMtu;
    __INOUT uint32_t *pMtuLen;
    __OUT char *pDns;
    __INOUT uint32_t *pDnsLen;
} ConnMgrModemIpv4Status_t;

typedef struct ConnMgrModemIpv6Status_s {
    __OUT char *pAddress;
    __INOUT uint32_t *pAddressLen;
    __OUT char *pGateway;
    __INOUT uint32_t *pGatewayLen;
    __OUT char *pMtu;
    __INOUT uint32_t *pMtuLen;
    __OUT char *pDns;
    __INOUT uint32_t *pDnsLen;
} ConnMgrModemIpv6Status_t;

typedef struct ConnMgrModemStatus_s {
    __OUT char *pConnected;
    __INOUT uint32_t *pConnectedLen;
    __OUT char *pInterface;
    __INOUT uint32_t *pInterfaceLen;
    __OUT char *pApn;
    __INOUT uint32_t *pApnLen;
    __OUT char *pRoaming;
    __INOUT uint32_t *pRoamingLen;
    __OUT struct ConnMgrModemIpv4Status_s *pIpv4Status;
    __OUT struct ConnMgrModemIpv6Status_s *pIpv6Status;
} ConnMgrModemStatus_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetStatus(
    __IN uint32_t Index,
    __OUT struct ConnMgrModemStatus_s *pStatus
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemUnlockSIMByPIN(
    __IN uint32_t Index, 
    __IN char *pPinCode
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSetAPN(
    __IN char *pApn
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSetUser(
    __IN char *pUser
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSetPassword(
    __IN char *pPassword
    );

/* IP Type */
#define ASUS_CMM_IP_TYPE_IPV4      0
#define ASUS_CMM_IP_TYPE_IPV6      1
#define ASUS_CMM_IP_TYPE_IPV4V6    2

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSetIPType(
    __IN uint32_t IpType
    );

typedef struct ConnMgrModemCarrierMcc_s {
    char *pMcc;
    uint32_t *pMccLen;
} ConnMgrModemCarrierMcc_t;

typedef struct ConnMgrModemCarrierMnc_s {
    char *pMnc;
    uint32_t *pMncLen;
} ConnMgrModemCarrierMnc_t;

typedef struct ConnMgrModemCarrierName_s {
    char *pOperatorName;
    uint32_t *pOperatorNameLen;
} ConnMgrModemCarrierName_t;

typedef struct ConnMgrModemCarrierInfo_s {
    struct ConnMgrModemCarrierMcc_s *pMcc;
    struct ConnMgrModemCarrierMnc_s *pMnc;
    struct ConnMgrModemCarrierName_s *pName;
} ConnMgrModemCarrierInfo_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemCheckCarrier(
    __IN uint32_t Index,
    __OUT struct ConnMgrModemCarrierInfo_s *pCarrierInfo
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSwitchCarrier(
    __IN uint32_t Index,
    __IN struct ConnMgrModemCarrierMcc_s *pMcc,
    __IN struct ConnMgrModemCarrierMnc_s *pMnc
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetICCID(
    __IN uint32_t Index,
    __OUT char *pBuffer ,
    __INOUT uint32_t *pBufLen
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetIMSI(
    __IN uint32_t Index,
    __OUT char *pBuffer ,
    __INOUT uint32_t *pBufLen
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetSignalStrength(
    __IN uint32_t Index,
    __OUT uint32_t *pValue
    );

typedef struct ConnMgrModemEvdoSignalInfo_s {
    float rssi;
    float ecio;
    float sinr;
    float io;
    uint32_t validFlag;
} ConnMgrModemEvdoSignalInfo_t;

typedef struct ConnMgrModemGsmSignalInfo_s {
    float rssi;
    uint32_t validFlag;
} ConnMgrModemGsmSignalInfo_t
;
typedef struct ConnMgrModemUmtsSignalInfo_s {
    float rssi;
    float rscp;
    float ecio;
    uint32_t validFlag;
} ConnMgrModemUmtsSignalInfo_t;

typedef struct ConnMgrModemLteSignalInfo_s {
    float rssi;
    float rsrq;
    float rsrp;
    float snr;
    uint32_t validFlag;
} ConnMgrModemLteSignalInfo_t;

typedef struct ConnMgrModemAdvSignalInfo_s {
    struct ConnMgrModemEvdoSignalInfo_s Evdo;
    struct ConnMgrModemGsmSignalInfo_s Gsm;
    struct ConnMgrModemUmtsSignalInfo_s Umts;
    struct ConnMgrModemLteSignalInfo_s Lte;
} ConnMgrModemAdvSignalInfo_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetAdvancedSignalInfo(
    __IN uint32_t Index,
    __OUT struct ConnMgrModemAdvSignalInfo_s *pAdvSignalInfo
    );


typedef struct ConnMgrModemProfile_s {
    __OUT char *pApn;
    __INOUT uint32_t *pApnLen;
    __OUT char *pUser;
    __INOUT uint32_t *pUserLen;
    __OUT char *pPassword;
    __INOUT uint32_t *pPasswordLen;
    __OUT char *pIpv4Method;
    __INOUT uint32_t *pIpv4MethodLen;
    __OUT char *pIpv6Method;
    __INOUT uint32_t *pIpv6MethodLen;
} ConnMgrModemProfile_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetProfile(
    __OUT struct ConnMgrModemProfile_s *pProfile
    );

typedef struct ConnMgrModemCellLocationInfo_s {
    __OUT char *pOperatorCode;
    __INOUT uint32_t *pOperatorCodeLen;
    __OUT char *pOperatorName;
    __INOUT uint32_t *pOperatorNameLen;
    __OUT char *pLocationAreaCode;
    __INOUT uint32_t *pLocationAreaCodeLen;
    __OUT char *pTrackingAreaCode;
    __INOUT uint32_t *pTrackingAreaCodeLen;
    __OUT char *pCellId;
    __INOUT uint32_t *pCellIdLen;
} ConnMgrModemCellLocationInfo_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemResetProfile();

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetCellLocationInfo(
    __IN uint32_t Index,
    __OUT struct ConnMgrModemCellLocationInfo_s *pCellLocInfo
    );

/* Keep Alive Status */
#define ASUS_CMM_KEEP_ALIVE_ON    1
#define ASUS_CMM_KEEP_ALIVE_OFF   0

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemSetKeepAlive(__IN uint32_t Enable);

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrModemGetKeepAliveStatus(
    __OUT uint32_t *pStatus
    );

/* Failover Status */
#define ASUS_CM_FAILOVER_ON     1
#define ASUS_CM_FAILOVER_OFF    0

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrSetFailover(__IN uint32_t Enable);

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrGetFailoverStatus(
    __OUT uint32_t *pStatus
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrSetFailoverGroup(
    __IN char **pInterface,
    __IN uint32_t InterfaceCnt
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusConnMgrGetFailoverGroup(
    __OUT char **pInterface,
    __INOUT uint32_t *ByteCntOfRow, //the number of interface
    __IN uint32_t ByteCntOfColumn
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorAvailable(__OUT char *pSupported);

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorEnable();

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorDisable();

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorGetStatus(__OUT char *pStatus);

typedef struct VcpGSensorValue_s {
    int x_mg;
    int y_mg;
    int z_mg;
} VcpGSensorValue_t;

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorRead(
    __OUT struct VcpGSensorValue_s *pGsensorValue
    );

typedef enum {
    VCP_GSENSOR_RES_2G,
    VCP_GSENSOR_RES_4G,
    VCP_GSENSOR_RES_8G,
    VCP_GSENSOR_RES_16G
} VCP_GSENSOR_RES;

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorGetResolution(
    __OUT VCP_GSENSOR_RES *pGsensorRes
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorSetResolution(
    __IN VCP_GSENSOR_RES GsensorRes
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorGetOffset(
    __OUT struct VcpGSensorValue_s *pGsensorValue
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorSetOffset(
    __IN struct VcpGSensorValue_s GsensorValue
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorGetAlarmThreshold(
    __OUT VCP_GSENSOR_RES *pGsensorRes,
    __OUT int *pValue
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorSetAlarmThreshold(
    __IN VCP_GSENSOR_RES GsensorRes,
    __IN int Value
    );

typedef enum {
    VCP_GSENSOR_ALARM_DISABLE                = 0x00000000,
    VCP_GSENSOR_ALARM_ENABLE_X               = 0x00000001,
    VCP_GSENSOR_ALARM_ENABLE_Y               = 0x00000002,
    VCP_GSENSOR_ALARM_ENABLE_X_Y             = 0x00000003,
    VCP_GSENSOR_ALARM_ENABLE_Z               = 0x00000004,
    VCP_GSENSOR_ALARM_ENABLE_X_Z             = 0x00000005,
    VCP_GSENSOR_ALARM_ENABLE_Y_Z             = 0x00000006,
    VCP_GSENSOR_ALARM_ENABLE_X_Y_Z           = 0x00000007
} VCP_VCP_GSENSOR_ALARM_ENABLE_SETTING;

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorGetAlarm(
    __OUT VCP_VCP_GSENSOR_ALARM_ENABLE_SETTING *pEnable
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGSensorSetAlarm(
    __IN VCP_VCP_GSENSOR_ALARM_ENABLE_SETTING Enable
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpRtcTimeSet(
    __IN uint32_t Year,
    __IN uint32_t Month,
    __IN uint32_t DayofMonth,
    __IN uint32_t Hour,
    __IN uint32_t Minute,
    __IN uint32_t Second,
    __IN uint32_t DayofWeek
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpRtcTimeGet(
    __OUT uint32_t *pYear,
    __OUT uint32_t *pMonth,
    __OUT uint32_t *pDayofMonth,
    __OUT uint32_t *pHour,
    __OUT uint32_t *pMinute,
    __OUT uint32_t *pSecond,
    __OUT uint32_t *pDayofWeek
    );

typedef enum {
    VCP_COM_PORT_MODE_RS232 = 232,
    VCP_COM_PORT_MODE_RS422 = 422,
    VCP_COM_PORT_MODE_RS485 = 485
} VCP_COM_PORT_MODE;

typedef enum {
    VCP_COM_PORT_BAUD_RATE_2400 = 2400,
    VCP_COM_PORT_BAUD_RATE_4800 = 4800,
    VCP_COM_PORT_BAUD_RATE_9600 = 9600,
    VCP_COM_PORT_BAUD_RATE_19200 = 19200,
    VCP_COM_PORT_BAUD_RATE_38400 = 38400,
    VCP_COM_PORT_BAUD_RATE_57600 = 57600,
    VCP_COM_PORT_BAUD_RATE_115200 = 115200
} VCP_COM_PORT_BAUD_RATE;

typedef enum {
    VCP_COM_PORT_PARITY_NONE,
    VCP_COM_PORT_PARITY_EVEN,
    VCP_COM_PORT_PARITY_ODD
} VCP_COM_PORT_PARITY;

typedef enum {
    VCP_COM_PORT_DATA_7_BIT,
    VCP_COM_PORT_DATA_8_BIT,
    VCP_COM_PORT_DATA_9_BIT
} VCP_COM_PORT_DATA_WIDTH;

typedef enum {
    VCP_COM_PORT_STOP_0_5_BIT,
    VCP_COM_PORT_STOP_1_BIT,
    VCP_COM_PORT_STOP_1_5_BIT,
    VCP_COM_PORT_STOP_2_BIT
} VCP_COM_PORT_STOP_BIT;

typedef enum {
    VCP_COM_PORT_FLOW_CONTROL_DISABLED,
    VCP_COM_PORT_FLOW_CONTROL_ENABLED
} VCP_COM_PORT_FLOW_CONTROL;

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpComPortInit(
    __IN uint32_t Id,
    __IN VCP_COM_PORT_MODE Mode,
    __IN VCP_COM_PORT_BAUD_RATE BaudRate,
    __IN VCP_COM_PORT_PARITY Parity,
    __IN VCP_COM_PORT_DATA_WIDTH DataWidth,
    __IN VCP_COM_PORT_STOP_BIT StopBit,
    __IN VCP_COM_PORT_FLOW_CONTROL FlowControl
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpComPortEnable(
    __IN uint32_t Id
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpComPortDisable(
    __IN uint32_t Id
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpComPortRead(
    __IN uint32_t Id,
    __OUT void *pBuffer,    /* Transfer Data pBuffer */
    __IN uint32_t BufLen,   /* Data pBuffer Length */
    __IN uint32_t ByteCnt   /* Byte Count to read */
    );

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpComPortWrite(
    __IN uint32_t Id,
    __IN void *pBuffer,    /* Transfer Data pBuffer */
    __IN uint32_t ByteCnt  /* Byte Count to write */
    );

/* Wake up */
typedef enum {
    VCP_WAKEUP_TYPE_DI0          = 1 << 0,
    VCP_WAKEUP_TYPE_DI1          = 1 << 1,
    VCP_WAKEUP_TYPE_DI2          = 1 << 2,
    VCP_WAKEUP_TYPE_DI3          = 1 << 3,
    VCP_WAKEUP_TYPE_IGNITION_OFF = 1 << 4,
    VCP_WAKEUP_TYPE_IGNITION_ON  = 1 << 5,
    VCP_WAKEUP_TYPE_GSENSOR      = 1 << 6,
    VCP_WAKEUP_TYPE_LTE          = 1 << 7,
    VCP_WAKEUP_TYPE_PMIC         = 1 << 8
} VCP_WAKEUP_TYPE;

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpWakeupTypeEnable(__IN VCP_WAKEUP_TYPE WakeupType);

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpWakeupTypeDisable(__IN VCP_WAKEUP_TYPE WakeupType);

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGetEnabledWakeupTypes(__OUT uint32_t *pWakeupType);

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpGetLastWakeupSource(__OUT uint32_t *pWakeupType);

typedef enum {
    VCP_POWER_MGMT_EVENT_POWER_ON = 1,
    VCP_POWER_MGMT_EVENT_POWER_OFF = 2
} VCP_POWER_MGMT_EVENT;

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpPowerMgmtSetEventDelay(__IN VCP_POWER_MGMT_EVENT EventType, __IN uint32_t Second);

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpPowerMgmtGetEventDelay(__IN VCP_POWER_MGMT_EVENT EventType, __OUT uint32_t *pSecond);

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpPowerMgmtSetLowBatteryProtectionThreshold(__IN float Voltage);

AsusStatus_t 
ASUS_CALLTYPE 
AsusVcpPowerMgmtGetLowBatteryProtectionThreshold(__OUT float *pVoltage);


/****************************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* _EAPI_H_ */
